"""
This type stub file was generated by pyright.
"""

"""Hooks for IPython.

In Python, it is possible to overwrite any method of any object if you really
want to.  But IPython exposes a few 'hooks', methods which are *designed* to
be overwritten by users for customization purposes.  This module defines the
default versions of all such hooks, which get used by IPython if not
overridden by the user.

Hooks are simple functions, but they should be declared with ``self`` as their
first argument, because when activated they are registered into IPython as
instance methods. The self argument will be the IPython running instance
itself, so hooks have full access to the entire IPython object.

If you wish to define a new hook and activate it, you can make an :doc:`extension
</config/extensions/index>` or a :ref:`startup script <startup_files>`. For
example, you could use a startup file like this::

    import os

    def calljed(self,filename, linenum):
        "My editor hook calls the jed editor directly."
        print "Calling my own editor, jed ..."
        if os.system('jed +%d %s' % (linenum,filename)) != 0:
            raise TryNext()

    def load_ipython_extension(ip):
        ip.set_hook('editor', calljed)

"""
__all__ = ["editor", "synchronize_with_editor", "show_in_pager", "pre_prompt_hook", "clipboard_get"]
deprecated = ...
def editor(self, filename, linenum=..., wait=...): # -> None:
    """Open the default editor at the given filename and linenumber.

    This is IPython's default editor hook, you can use it as an example to
    write your own modified one.  To set your own editor function as the
    new editor hook, call ip.set_hook('editor',yourfunc)."""
    ...

def synchronize_with_editor(self, filename, linenum, column): # -> None:
    ...

class CommandChainDispatcher:
    """ Dispatch calls to a chain of commands until some func can handle it

    Usage: instantiate, execute "add" to add commands (with optional
    priority), execute normally via f() calling mechanism.

    """
    def __init__(self, commands=...) -> None:
        ...
    
    def __call__(self, *args, **kw):
        """ Command chain is called just like normal func.

        This will call all funcs in chain with the same args as were given to
        this function, and return the result of first func that didn't raise
        TryNext"""
        ...
    
    def __str__(self) -> str:
        ...
    
    def add(self, func, priority=...): # -> None:
        """ Add a func to the cmd chain with given priority """
        ...
    
    def __iter__(self):
        """ Return all objects in chain.

        Handy if the objects are not callable.
        """
        ...
    


def show_in_pager(self, data, start, screen_lines):
    """ Run a string through pager """
    ...

def pre_prompt_hook(self): # -> None:
    """ Run before displaying the next prompt

    Use this e.g. to display output from asynchronous operations (in order
    to not mess up text entry)
    """
    ...

def clipboard_get(self):
    """ Get text from the clipboard.
    """
    ...




from typing import Literal
from numpy.random import RandomState
from ._svmlight_format_io import load_svmlight_files as load_svmlight_files
from os import remove as remove, makedirs as makedirs
from os.path import exists as exists, join as join
from ._base import RemoteFileMetadata as RemoteFileMetadata, load_descr as load_descr
from gzip import GzipFile as GzipFile
from .._typing import Int
from ..utils import shuffle as shuffle_, Bunch
from . import get_data_home as get_data_home

# Author: Tom Dupre la Tour
# License: BSD 3 clause

import logging

import numpy as np
import scipy.sparse as sp
import joblib


# The original vectorized data can be found at:
#    http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a13-vector-files/lyrl2004_vectors_test_pt0.dat.gz
#    http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a13-vector-files/lyrl2004_vectors_test_pt1.dat.gz
#    http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a13-vector-files/lyrl2004_vectors_test_pt2.dat.gz
#    http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a13-vector-files/lyrl2004_vectors_test_pt3.dat.gz
#    http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/a13-vector-files/lyrl2004_vectors_train.dat.gz
# while the original stemmed token files can be found
# in the README, section B.12.i.:
#    http://www.ai.mit.edu/projects/jmlr/papers/volume5/lewis04a/lyrl2004_rcv1v2_README.htm
XY_METADATA = ...

# The original data can be found at:
# http://jmlr.csail.mit.edu/papers/volume5/lewis04a/a08-topic-qrels/rcv1-v2.topics.qrels.gz
TOPICS_METADATA = ...

logger = ...


def fetch_rcv1(
    *,
    data_home: None | str = None,
    subset: Literal["train", "test", "all", "all"] = "all",
    download_if_missing: bool = True,
    random_state: RandomState | None | Int = None,
    shuffle: bool = False,
    return_X_y: bool = False,
) -> tuple[Bunch, tuple]:
    ...



from vispy.util.event import Event
from typing import Literal
from vispy.color import Colormap, Color

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np
from .widget import Widget
from ...visuals import ColorBarVisual

class ColorBarWidget(Widget):
    def __init__(
        self,
        cmap: str | Colormap,
        orientation: Literal["left", "right", "top", "bottom"],
        label: str = "",
        label_color: str | Color = "black",
        clim: tuple[float, float] = ...,
        border_width: float = 0.0,
        border_color: str | Color = "black",
        padding: tuple = ...,
        axis_ratio: float = 0.05,
        **kwargs,
    ): ...
    def on_resize(self, event: Event): ...
    def _update_colorbar(self): ...
    def _calc_size(self): ...
    @property
    def cmap(self): ...
    @cmap.setter
    def cmap(self, cmap): ...
    @property
    def label(self): ...
    @label.setter
    def label(self, label): ...
    @property
    def ticks(self): ...
    @ticks.setter
    def ticks(self, ticks): ...
    @property
    def clim(self): ...
    @clim.setter
    def clim(self, clim): ...
    @property
    def border_color(self): ...
    @border_color.setter
    def border_color(self, border_color): ...
    @property
    def border_width(self): ...
    @border_width.setter
    def border_width(self, border_width): ...
    @property
    def orientation(self): ...



"""
This type stub file was generated by pyright.
"""

"""Logger class for IPython's logging facilities.
"""
class Logger:
    """A Logfile class with different policies for file creation"""
    def __init__(self, home_dir, logfname=..., loghead=..., logmode=...) -> None:
        ...
    
    logmode = ...
    def logstart(self, logfname=..., loghead=..., logmode=..., log_output=..., timestamp=..., log_raw_input=...): # -> None:
        """Generate a new log-file with a default header.

        Raises RuntimeError if the log has already been started"""
        ...
    
    def switch_log(self, val): # -> None:
        """Switch logging on/off. val should be ONLY a boolean."""
        ...
    
    def logstate(self): # -> None:
        """Print a status message about the logger."""
        ...
    
    def log(self, line_mod, line_ori): # -> None:
        """Write the sources to a log.

        Inputs:

        - line_mod: possibly modified input, such as the transformations made
          by input prefilters or input handlers of various kinds. This should
          always be valid Python.

        - line_ori: unmodified input line from the user. This is not
          necessarily valid Python.
        """
        ...
    
    def log_write(self, data, kind=...): # -> None:
        """Write data to the log file, if active"""
        ...
    
    def logstop(self): # -> None:
        """Fully stop logging and close log file.

        In order to start logging again, a new logstart() call needs to be
        made, possibly (though not necessarily) with a new filename, mode and
        other options."""
        ...
    
    close_log = ...





"""
This type stub file was generated by pyright.
"""

"""Global IPython app to support test running.

We must start our own ipython object and heavily muck with it so that all the
modifications IPython makes to system behavior don't send the doctest machinery
into a fit.  This code should be considered a gross hack, but it gets the job
done.
"""
def get_ipython(): # -> None:
    ...

def xsys(self, cmd): # -> None:
    """Replace the default system call with a capturing one for doctest.
    """
    ...

def start_ipython(): # -> None:
    """Start a global IPython shell, which we need for IPython-specific syntax.
    """
    ...




# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from OpenGL import GL as _GL
from . import _pyopengl2
from . import _constants

def _inject(): ...

# List of deprecated functions, obtained by parsing gl.spec
_deprecated_functions: str = ...



  
class LossFunction:
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y) -> float: ...


class Regression(LossFunction):
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y) -> float: ...


class Classification(LossFunction):
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y) -> float: ...

    
class ModifiedHuber(Classification):
    """Modified Huber loss for binary classification with y in {-1, 1}

    This is equivalent to quadratically smoothed SVM with gamma = 2.

    See T. Zhang 'Solving Large Scale Linear Prediction Problems Using
    Stochastic Gradient Descent', ICML'04.
    """
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...

    
class Hinge(Classification):
    """Hinge loss for binary classification tasks with y in {-1,1}

    Parameters
    ----------

    threshold : float > 0.0
        Margin threshold. When threshold=1.0, one gets the loss used by SVM.
        When threshold=0.0, one gets the loss used by the Perceptron.
    """
    threshold: float
    def __init__(self, threshold: float=1.0) -> None: ...
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...


class SquaredHinge(Classification):
    """Squared Hinge loss for binary classification tasks with y in {-1,1}

    Parameters
    ----------

    threshold : float > 0.0
        Margin threshold. When threshold=1.0, one gets the loss used by
        (quadratically penalized) SVM.
    """
    threshold: float
    def __init__(self, threshold: float=1.0) -> None: ...
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...


class Log(Classification):
    """Logistic regression loss for binary classification with y in {-1, 1}"""
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...

    
class SquaredLoss(Regression):
    """Squared loss traditional used in linear regression."""
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...


class Huber(Regression):
    """Huber regression loss

    Variant of the SquaredLoss that is robust to outliers (quadratic near zero,
    linear in for large errors).

    https://en.wikipedia.org/wiki/Huber_Loss_Function
    """
    c: float
    def __init__(self, c: float) -> None: ...
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...


class EpsilonInsensitive(Regression):
    """Epsilon-Insensitive loss (used by SVR).

    loss = max(0, |y - p| - epsilon)
    """
    epsilon: float
    def __init__(self, epsilon: float) -> None: ...
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...


class SquaredEpsilonInsensitive(Regression):
    """Epsilon-Insensitive loss.

    loss = max(0, |y - p| - epsilon)^2
    """
    epsilon: float
    def __init__(self, epsilon: float) -> None: ...
    def loss(self, p: float, y: float) -> float: ...
    def dloss(self, p: float, y: float) -> float: ...



from typing import Sequence
from threadpoolctl import threadpool_limits as threadpool_limits
from .utils.fixes import parse_version as parse_version
from _pytest.doctest import DoctestItem as DoctestItem
from os import environ as environ
from .tests import random_seed as random_seed
from ._min_dependencies import PYTEST_MIN_VERSION as PYTEST_MIN_VERSION
from functools import wraps as wraps
from .datasets import (
    fetch_20newsgroups as fetch_20newsgroups,
    fetch_20newsgroups_vectorized as fetch_20newsgroups_vectorized,
    fetch_california_housing as fetch_california_housing,
    fetch_covtype as fetch_covtype,
    fetch_kddcup99 as fetch_kddcup99,
    fetch_olivetti_faces as fetch_olivetti_faces,
    fetch_rcv1 as fetch_rcv1,
)
import platform
import sys

import pytest
import numpy as np

dataset_fetchers: dict = ...

_SKIP32_MARK = ...


# Global fixtures
def global_dtype(request):
    ...


# Adds fixtures for fetching data
fetch_20newsgroups_fxt = ...
fetch_20newsgroups_vectorized_fxt = ...
fetch_california_housing_fxt = ...
fetch_covtype_fxt = ...
fetch_kddcup99_fxt = ...
fetch_olivetti_faces_fxt = ...
fetch_rcv1_fxt = ...


def pytest_collection_modifyitems(config, items: Sequence):
    ...


def pyplot():
    ...


def pytest_runtest_setup(item):
    ...


def pytest_configure(config):
    ...



from typing import (
    Any,
    Dict,
    Iterator,
    List,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
    Union,
)

from django.apps import AppConfig
from django.apps.registry import Apps
from django.db.models.base import Model
from django.db.models.fields import Field
from django.db.models.manager import Manager

class AppConfigStub(AppConfig): ...

class ModelState:
    name: str
    app_label: str
    fields: List[Tuple[str, Field[Any, Any]]]
    options: Dict[str, Any] = ...
    bases: Tuple[Type[Model]] = ...
    managers: List[Tuple[str, Manager[Any]]] = ...
    def __init__(
        self,
        app_label: str,
        name: str,
        fields: List[Tuple[str, Field[Any, Any]]],
        options: Optional[Dict[str, Any]] = ...,
        bases: Optional[Sequence[Union[Type[Model], str]]] = ...,
        managers: Optional[List[Tuple[str, Manager[Any]]]] = ...,
    ) -> None: ...
    def clone(self) -> ModelState: ...
    def construct_managers(self) -> Iterator[Tuple[str, Manager[Any]]]: ...
    @classmethod
    def from_model(cls, model: Type[Model], exclude_rels: bool = ...) -> ModelState: ...
    def get_field_by_name(self, name: str) -> Field[Any, Any]: ...
    @property
    def name_lower(self) -> str: ...
    def render(self, apps: Apps) -> Any: ...

def get_related_models_tuples(model: Type[Model]) -> Set[Tuple[str, str]]: ...
def get_related_models_recursive(model: Type[Model]) -> Set[Tuple[str, str]]: ...

class ProjectState:
    is_delayed: bool
    models: Dict[Any, Any]
    real_apps: List[str]
    def __init__(
        self,
        models: Optional[Dict[Tuple[str, str], ModelState]] = ...,
        real_apps: Optional[List[str]] = ...,
    ) -> None: ...
    def add_model(self, model_state: ModelState) -> None: ...
    @property
    def apps(self) -> StateApps: ...
    def clear_delayed_apps_cache(self) -> None: ...
    def clone(self) -> ProjectState: ...
    @property
    def concrete_apps(self) -> StateApps: ...
    @classmethod
    def from_apps(cls, apps: Apps) -> ProjectState: ...
    def reload_model(
        self, app_label: str, model_name: str, delay: bool = ...
    ) -> None: ...
    def reload_models(self, models: List[Any], delay: bool = ...) -> None: ...
    def remove_model(self, app_label: str, model_name: str) -> None: ...

class StateApps(Apps):
    real_models: List[ModelState]
    def __init__(
        self,
        real_apps: List[str],
        models: Dict[Tuple[str, str], ModelState],
        ignore_swappable: bool = ...,
    ) -> None: ...
    def bulk_update(self) -> Iterator[None]: ...
    def clone(self) -> StateApps: ...
    def render_multiple(self, model_states: List[ModelState]) -> None: ...
    def unregister_model(self, app_label: str, model_name: str) -> None: ...



from typing import (
    IO,
    Iterable,
)

from pandas.io.formats.format import (
    DataFrameFormatter as DataFrameFormatter,
    TableFormatter as TableFormatter,
)

class HTMLFormatter:
    indent_delta: int = ...
    fmt = ...
    classes = ...
    frame = ...
    columns = ...
    elements = ...
    bold_rows = ...
    escape = ...
    show_dimensions = ...
    border = ...
    table_id = ...
    render_links = ...
    def __init__(
        self,
        formatter: DataFrameFormatter,
        classes: str | list[str] | tuple[str, ...] | None = ...,
        border: int | None = ...,
    ) -> None: ...
    @property
    def show_row_idx_names(self) -> bool: ...
    @property
    def show_col_idx_names(self) -> bool: ...
    @property
    def row_levels(self) -> int: ...
    @property
    def is_truncated(self) -> bool: ...
    @property
    def ncols(self) -> int: ...
    def write(self, s, indent: int = ...) -> None: ...
    def write_th(
        self, s, header: bool = ..., indent: int = ..., tags: str | None = ...
    ) -> None: ...
    def write_td(self, s, indent: int = ..., tags: str | None = ...) -> None: ...
    def write_tr(
        self,
        line: Iterable,
        indent: int = ...,
        indent_delta: int = ...,
        header: bool = ...,
        align: str | None = ...,
        tags: dict[int, str] | None = ...,
        nindex_levels: int = ...,
    ) -> None: ...
    def render(self) -> list[str]: ...
    def write_result(self, buf: IO[str]) -> None: ...

class NotebookFormatter(HTMLFormatter):
    def write_style(self) -> None: ...
    def render(self) -> list[str]: ...



from vispy.util.svg.color import Color
from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from ..geometry import create_sphere
from .mesh import MeshVisual
from .visual import CompoundVisual

class SphereVisual(CompoundVisual):
    def __init__(
        self,
        radius: float = 1.0,
        cols: int = 30,
        rows: int = 30,
        depth: int = 30,
        subdivisions: int = 3,
        method: str = "latitude",
        vertex_colors: NDArray | None = None,
        face_colors: NDArray | None = None,
        color: Color = ...,
        edge_color: tuple | Color | None = None,
        shading: str | None = None,
        **kwargs,
    ): ...
    @property
    def mesh(self): ...
    @property
    def border(self): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy._lib._test_ccallback, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def test_call_nodata() -> typing.Any:
    ...

def test_call_nonlocal() -> typing.Any:
    ...

def test_call_simple() -> typing.Any:
    ...

def test_get_data_capsule() -> typing.Any:
    ...

def test_get_plus1_capsule() -> typing.Any:
    ...

def test_get_plus1b_capsule() -> typing.Any:
    ...

def test_get_plus1bc_capsule() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




from skimage.future.graph.rag import RAG
from numpy.typing import NDArray
import networkx as nx
import numpy as np
from scipy import sparse

def DW_matrices(graph: RAG): ...
def ncut_cost(cut: NDArray, D, W) -> float: ...



from numpy.typing import ArrayLike, NDArray
from itertools import product
import numpy as np

from ..measure.pnpoly import grid_points_in_poly

from ..measure._label import label

from .._shared.utils import warn

__all__ = ["convex_hull_image", "convex_hull_object"]

def _offsets_diamond(ndim): ...
def _check_coords_in_hull(gridcoords, hull_equations, tolerance): ...
def convex_hull_image(
    image: ArrayLike, offset_coordinates: bool = True, tolerance: float = 1e-10
): ...
def convex_hull_object(image, *, connectivity=2) -> NDArray: ...



from collections.abc import (
    Callable,
    Hashable,
    Mapping,
    Sequence,
)
import datetime
from typing import (
    Literal,
    overload,
)

import numpy as np
import pandas as pd
from pandas.core.frame import DataFrame
from pandas.core.groupby.grouper import Grouper
from pandas.core.indexes.base import Index
from pandas.core.series import Series
from typing_extensions import TypeAlias

from pandas._typing import (
    AnyArrayLike,
    ArrayLike,
    HashableT1,
    HashableT2,
    HashableT3,
    Label,
    Scalar,
    ScalarT,
    npt,
)

_PivotAggCallable: TypeAlias = Callable[[Series], ScalarT]

_PivotAggFunc: TypeAlias = (
    _PivotAggCallable
    | np.ufunc
    | Literal["mean", "sum", "count", "min", "max", "median", "std", "var"]
)

_NonIterableHashable: TypeAlias = (
    str
    | datetime.date
    | datetime.datetime
    | datetime.timedelta
    | bool
    | int
    | float
    | complex
    | pd.Timestamp
    | pd.Timedelta
)

_PivotTableIndexTypes: TypeAlias = Label | list[HashableT1] | Series | Grouper | None
_PivotTableColumnsTypes: TypeAlias = Label | list[HashableT2] | Series | Grouper | None

_ExtendedAnyArrayLike: TypeAlias = AnyArrayLike | ArrayLike

@overload
def pivot_table(
    data: DataFrame,
    values: Label | list[HashableT3] | None = ...,
    index: _PivotTableIndexTypes = ...,
    columns: _PivotTableColumnsTypes = ...,
    aggfunc: _PivotAggFunc
    | list[_PivotAggFunc]
    | Mapping[Hashable, _PivotAggFunc] = ...,
    fill_value: Scalar | None = ...,
    margins: bool = ...,
    dropna: bool = ...,
    margins_name: str = ...,
    observed: bool = ...,
    sort: bool = ...,
) -> DataFrame: ...

# Can only use Index or ndarray when index or columns is a Grouper
@overload
def pivot_table(
    data: DataFrame,
    values: Label | list[HashableT3] | None = ...,
    *,
    index: Grouper,
    columns: _PivotTableColumnsTypes | Index | npt.NDArray = ...,
    aggfunc: _PivotAggFunc
    | list[_PivotAggFunc]
    | Mapping[Hashable, _PivotAggFunc] = ...,
    fill_value: Scalar | None = ...,
    margins: bool = ...,
    dropna: bool = ...,
    margins_name: str = ...,
    observed: bool = ...,
    sort: bool = ...,
) -> DataFrame: ...
@overload
def pivot_table(
    data: DataFrame,
    values: Label | list[HashableT3] | None = ...,
    index: _PivotTableIndexTypes | Index | npt.NDArray = ...,
    *,
    columns: Grouper,
    aggfunc: _PivotAggFunc
    | list[_PivotAggFunc]
    | Mapping[Hashable, _PivotAggFunc] = ...,
    fill_value: Scalar | None = ...,
    margins: bool = ...,
    dropna: bool = ...,
    margins_name: str = ...,
    observed: bool = ...,
    sort: bool = ...,
) -> DataFrame: ...
def pivot(
    data: DataFrame,
    *,
    index: _NonIterableHashable | list[HashableT1] = ...,
    columns: _NonIterableHashable | list[HashableT2] = ...,
    values: _NonIterableHashable | list[HashableT3] = ...,
) -> DataFrame:
    """
Return reshaped DataFrame organized by given index / column values.

Reshape data (produce a "pivot" table) based on column values. Uses
unique values from specified `index` / `columns` to form axes of the
resulting DataFrame. This function does not support data
aggregation, multiple values will result in a MultiIndex in the
columns. See the :ref:`User Guide <reshaping>` for more on reshaping.

Parameters
----------
data : DataFrame
columns : str or object or a list of str
    Column to use to make new frame's columns.

    .. versionchanged:: 1.1.0
       Also accept list of columns names.

index : str or object or a list of str, optional
    Column to use to make new frame's index. If not given, uses existing index.

    .. versionchanged:: 1.1.0
       Also accept list of index names.

values : str, object or a list of the previous, optional
    Column(s) to use for populating new frame's values. If not
    specified, all remaining columns will be used and the result will
    have hierarchically indexed columns.

Returns
-------
DataFrame
    Returns reshaped DataFrame.

Raises
------
ValueError:
    When there are any `index`, `columns` combinations with multiple
    values. `DataFrame.pivot_table` when you need to aggregate.

See Also
--------
DataFrame.pivot_table : Generalization of pivot that can handle
    duplicate values for one index/column pair.
DataFrame.unstack : Pivot based on the index values instead of a
    column.
wide_to_long : Wide panel to long format. Less flexible but more
    user-friendly than melt.

Notes
-----
For finer-tuned control, see hierarchical indexing documentation along
with the related stack/unstack methods.

Reference :ref:`the user guide <reshaping.pivot>` for more examples.

Examples
--------
>>> df = pd.DataFrame({'foo': ['one', 'one', 'one', 'two', 'two',
...                            'two'],
...                    'bar': ['A', 'B', 'C', 'A', 'B', 'C'],
...                    'baz': [1, 2, 3, 4, 5, 6],
...                    'zoo': ['x', 'y', 'z', 'q', 'w', 't']})
>>> df
    foo   bar  baz  zoo
0   one   A    1    x
1   one   B    2    y
2   one   C    3    z
3   two   A    4    q
4   two   B    5    w
5   two   C    6    t

>>> df.pivot(index='foo', columns='bar', values='baz')
bar  A   B   C
foo
one  1   2   3
two  4   5   6

>>> df.pivot(index='foo', columns='bar')['baz']
bar  A   B   C
foo
one  1   2   3
two  4   5   6

>>> df.pivot(index='foo', columns='bar', values=['baz', 'zoo'])
      baz       zoo
bar   A  B  C   A  B  C
foo
one   1  2  3   x  y  z
two   4  5  6   q  w  t

You could also assign a list of column names or a list of index names.

>>> df = pd.DataFrame({
...        "lev1": [1, 1, 1, 2, 2, 2],
...        "lev2": [1, 1, 2, 1, 1, 2],
...        "lev3": [1, 2, 1, 2, 1, 2],
...        "lev4": [1, 2, 3, 4, 5, 6],
...        "values": [0, 1, 2, 3, 4, 5]})
>>> df
    lev1 lev2 lev3 lev4 values
0   1    1    1    1    0
1   1    1    2    2    1
2   1    2    1    3    2
3   2    1    2    4    3
4   2    1    1    5    4
5   2    2    2    6    5

>>> df.pivot(index="lev1", columns=["lev2", "lev3"], values="values")
lev2    1         2
lev3    1    2    1    2
lev1
1     0.0  1.0  2.0  NaN
2     4.0  3.0  NaN  5.0

>>> df.pivot(index=["lev1", "lev2"], columns=["lev3"], values="values")
      lev3    1    2
lev1  lev2
   1     1  0.0  1.0
         2  2.0  NaN
   2     1  4.0  3.0
         2  NaN  5.0

A ValueError is raised if there are any duplicates.

>>> df = pd.DataFrame({"foo": ['one', 'one', 'two', 'two'],
...                    "bar": ['A', 'A', 'B', 'C'],
...                    "baz": [1, 2, 3, 4]})
>>> df
   foo bar  baz
0  one   A    1
1  one   A    2
2  two   B    3
3  two   C    4

Notice that the first two rows are the same for our `index`
and `columns` arguments.

>>> df.pivot(index='foo', columns='bar', values='baz')
Traceback (most recent call last):
   ...
ValueError: Index contains duplicate entries, cannot reshape
    """
    pass
@overload
def crosstab(
    index: list | _ExtendedAnyArrayLike | list[Sequence | _ExtendedAnyArrayLike],
    columns: list | _ExtendedAnyArrayLike | list[Sequence | _ExtendedAnyArrayLike],
    values: list | _ExtendedAnyArrayLike,
    rownames: list[HashableT1] | None = ...,
    colnames: list[HashableT2] | None = ...,
    *,
    aggfunc: str | np.ufunc | Callable[[Series], float],
    margins: bool = ...,
    margins_name: str = ...,
    dropna: bool = ...,
    normalize: bool | Literal[0, 1, "all", "index", "columns"] = ...,
) -> DataFrame: ...
@overload
def crosstab(
    index: list | _ExtendedAnyArrayLike | list[Sequence | _ExtendedAnyArrayLike],
    columns: list | _ExtendedAnyArrayLike | list[Sequence | _ExtendedAnyArrayLike],
    values: None = ...,
    rownames: list[HashableT1] | None = ...,
    colnames: list[HashableT2] | None = ...,
    aggfunc: None = ...,
    margins: bool = ...,
    margins_name: str = ...,
    dropna: bool = ...,
    normalize: bool | Literal[0, 1, "all", "index", "columns"] = ...,
) -> DataFrame: ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier. All rights reserved.
# Distributed under the terms of the new BSD License.
# -----------------------------------------------------------------------------

import copy
from ...util import logger
from .path import Path
from .base import namespace
from .transformable import Transformable

class Group(Transformable):
    def __init__(self, content=None, parent=None): ...
    @property
    def flatten(self): ...
    @property
    def paths(self): ...
    def __repr__(self): ...
    @property
    def xml(self): ...
    def _xml(self, prefix=""): ...



import numpy as np
from typing import Any, Callable, Generator, Iterator, overload
from matplotlib._typing import *
from matplotlib.artist import Artist

import collections
import collections.abc
import contextlib

class _StrongRef:
    def __init__(self, obj) -> None: ...
    def __call__(self): ...
    def __eq__(self, other) -> bool: ...
    def __hash__(self) -> int: ...

class CallbackRegistry:
    def __init__(
        self, exception_handler: Callable = ..., *, signals: list = ...
    ) -> None: ...
    def __getstate__(self)-> dict[str, Any]: ...
    def __setstate__(self, state: dict[str, Any]) -> None: ...
    def connect(self, signal, func: Callable)-> int: ...
    def disconnect(self, cid: int)-> None: ...
    def process(self, s: str, *args, **kwargs)-> None: ...
    @contextlib.contextmanager
    def blocked(self, *, signal: str = ...)-> Generator: ...

class silent_list(list):
    def __init__(self, type, seq=...) -> None: ...
    def __repr__(self)-> str: ...

def strip_math(s: str)-> str: ...
def is_writable_file_like(obj)-> bool: ...
def file_requires_unicode(x)-> bool: ...
def to_filehandle(
    fname: str | PathLike | FileLike,
    flag: str = "r",
    return_opened: bool = False,
    encoding: str | None = None,
)-> tuple[FileLike, bool]: ...
def open_file_cm(path_or_file: str | PathLike | FileLike, mode=..., encoding=...)-> FileLike: ...
def is_scalar_or_string(val) -> bool: ...
@overload
def get_sample_data(
    fname, asfileobj: bool = ..., *, np_load: bool = ...
) -> ArrayLike | tuple | dict: ...
@overload
def get_sample_data(fname, asfileobj: bool, *, np_load=...) -> PathLike: ...
@overload
def get_sample_data(fname, asfileobj: bool = ..., *, np_load=...) -> FileLike: ...
def flatten(seq, scalarp=...)-> Generator: ...

class maxdict(dict):
    def __init__(self, maxsize) -> None: ...
    def __setitem__(self, k, v)-> None: ...

class Stack:
    def __init__(self, default=...) -> None: ...
    def __call__(self): ...
    def __len__(self)-> int: ...
    def __getitem__(self, ind): ...
    def forward(self): ...
    def back(self): ...
    def push(self, o): ...
    def home(self): ...
    def empty(self) -> bool: ...
    def clear(self)-> None: ...
    def bubble(self, o): ...
    def remove(self, o)-> None: ...

def report_memory(i=...): int: ...
def safe_masked_invalid(x, copy=...)-> np.ndarray: ...
def print_cycles(objects, outstream=..., show_progress: bool = ...)-> None: ...

class Grouper:
    def __init__(self, init=...) -> None: ...
    def __contains__(self, item)-> bool: ...
    def clean(self)-> None: ...
    def join(self, a, *args)-> bool: ...
    def joined(self, a, b)-> bool: ...
    def remove(self, a)-> None: ...
    def __iter__(self)-> Generator: ...
    def get_siblings(self, a)-> list: ...

class GrouperView:
    def __init__(self, grouper) -> None: ...

    class _GrouperMethodForwarder:
        def __init__(self, deprecated_kw=...) -> None: ...
        def __set_name__(self, owner, name): ...

    joined = ...
    get_siblings = ...
    clean = ...
    join = ...
    remove = ...

def simple_linear_interpolation(a, steps: int) -> np.ndarray: ...
def delete_masked_points(*args)-> tuple: ...
def boxplot_stats(
    X: ArrayLike,
    whis: float = 1.5,
    bootstrap: int = ...,
    labels: ArrayLike = ...,
    autorange: bool = False,
) -> list[dict]: ...

ls_mapper: dict[str, str] = ...
ls_mapper_r: dict[str, str] = ...

def contiguous_regions(mask)->list[tuple]: ...
def is_math_text(s) -> bool: ...
def violin_stats(
    X: ArrayLike, method: Callable, points: int = ..., quantiles: ArrayLike = ...
) -> list[dict]: ...
def pts_to_prestep(x: list, *args) -> list: ...
def pts_to_poststep(x: list, *args) -> list: ...
def pts_to_midstep(x: list, *args) -> list: ...

STEP_LOOKUP_MAP: dict = ...

def index_of(
    y: float | ArrayLike,
)-> tuple[np.ndarray, np.ndarray]: ...
def safe_first_element(obj): ...
def sanitize_sequence(data)-> list: ...
def normalize_kwargs(kw: dict | None, alias_mapping: dict | Artist = ...)-> dict: ...

class _OrderedSet(collections.abc.MutableSet):
    def __init__(self) -> None: ...
    def __contains__(self, key)-> bool: ...
    def __iter__(self)->Iterator: ...
    def __len__(self)-> int: ...
    def add(self, key)-> None: ...
    def discard(self, key)-> None: ...



from typing import Any, Dict, Iterable, Optional, TypeVar, Union

from django.db.models import Field
from django.db.models.expressions import (
    Combinable,
    CombinedExpression,
    Func,
    Value,
    _OutputField,
)
from django.db.models.fields import (
    _ErrorMessagesToOverride,
    _FieldChoices,
    _ValidatorCallable,
)
from django.db.models.lookups import Lookup

_Expression = Union[str, Combinable, "SearchQueryCombinable"]

class SearchVectorExact(Lookup[Any]): ...

class SearchVectorField(Field[Any, Any]):
    def __init__(
        self,
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...

class SearchQueryField(Field[Any, Any]):
    def __init__(
        self,
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...

class SearchVectorCombinable:
    ADD: str = ...

class SearchVector(SearchVectorCombinable, Func):
    config: Optional[Any] = ...
    def __init__(self, *expressions: _Expression, **extra: Any) -> None: ...

class CombinedSearchVector(SearchVectorCombinable, CombinedExpression):
    def __init__(
        self,
        lhs: Any,
        connector: Any,
        rhs: Any,
        config: Optional[_Expression] = ...,
        output_field: Optional[_OutputField] = ...,
    ) -> None: ...

_T = TypeVar("_T", bound="SearchQueryCombinable")

class SearchQueryCombinable:
    BITAND: str = ...
    BITOR: str = ...
    def __or__(self: _T, other: SearchQueryCombinable) -> _T: ...
    def __ror__(self: _T, other: SearchQueryCombinable) -> _T: ...
    def __and__(self: _T, other: SearchQueryCombinable) -> _T: ...
    def __rand__(self: _T, other: SearchQueryCombinable) -> _T: ...

class SearchQuery(SearchQueryCombinable, Value):  # type: ignore
    SEARCH_TYPES: Dict[str, str] = ...
    def __init__(
        self,
        value: str,
        output_field: Optional[_OutputField] = ...,
        *,
        config: Optional[_Expression] = ...,
        invert: bool = ...,
        search_type: str = ...
    ) -> None: ...
    def __invert__(self: _T) -> _T: ...

class CombinedSearchQuery(SearchQueryCombinable, CombinedExpression):  # type: ignore
    def __init__(
        self,
        lhs: Any,
        connector: Any,
        rhs: Any,
        config: Optional[_Expression] = ...,
        output_field: Optional[_OutputField] = ...,
    ) -> None: ...

class SearchRank(Func):
    def __init__(
        self,
        vector: Union[SearchVector, _Expression],
        query: Union[SearchQuery, _Expression],
        **extra: Any
    ) -> None: ...

class TrigramBase(Func):
    def __init__(self, expression: _Expression, string: str, **extra: Any) -> None: ...

class TrigramSimilarity(TrigramBase): ...
class TrigramDistance(TrigramBase): ...



import numpy as np
from typing import Sequence
from ._typing import *
from .transforms import Affine2D, Bbox, Transform

from functools import lru_cache

class Path:

    code_type = np.uint8
    STOP = code_type(0)
    MOVETO = code_type(1)
    LINETO = code_type(2)
    CURVE3 = code_type(3)
    CURVE4 = code_type(4)
    CLOSEPOLY = code_type(79)
    NUM_VERTICES_FOR_CODE = ...
    def __init__(
        self,
        vertices: ArrayLike,
        codes: ArrayLike | None = ...,
        _interpolation_steps: int = ...,
        closed: bool = ...,
        readonly: bool = ...,
    ) -> None: ...
    @property
    def vertices(self) -> np.ndarray: ...
    @vertices.setter
    def vertices(self, vertices: ArrayLike): ...
    @property
    def codes(self) -> np.ndarray: ...
    @codes.setter
    def codes(self, codes: ArrayLike): ...
    @property
    def simplify_threshold(self): ...
    @simplify_threshold.setter
    def simplify_threshold(self, threshold): ...
    @property
    def should_simplify(self) -> bool: ...
    @should_simplify.setter
    def should_simplify(self, should_simplify: bool): ...
    @property
    def readonly(self) -> bool: ...
    def copy(self) -> Path: ...
    def __deepcopy__(self, memo=...) -> Path: ...
    deepcopy = ...
    @classmethod
    def make_compound_path_from_polys(cls, XY): ...
    @classmethod
    def make_compound_path(cls, *args): ...
    def __repr__(self): ...
    def __len__(self): ...
    def iter_segments(
        self,
        transform: None = ...,
        remove_nans: bool = ...,
        clip: None | float | float = ...,
        snap: None | bool = ...,
        stroke_width: float = ...,
        simplify: None | bool = ...,
        curves: bool = ...,
        sketch: None | Sequence = ...,
    ): ...
    def iter_bezier(self, **kwargs): ...
    def cleaned(
        self,
        transform: Transform = ...,
        remove_nans=...,
        clip=...,
        *,
        simplify=...,
        curves=...,
        stroke_width: float = ...,
        snap=...,
        sketch=...
    ): ...
    def transformed(self, transform) -> Path: ...
    def contains_point(
        self,
        point: Sequence[float],
        transform: Transform = ...,
        radius: float = ...,
    ) -> bool: ...
    def contains_points(
        self, points: ArrayLike, transform: Transform = ..., radius: float = ...
    ) -> list[bool]: ...
    def contains_path(self, path: Path, transform: Transform = ...) -> bool: ...
    def get_extents(self, transform: Transform = ..., **kwargs) -> Bbox: ...
    def intersects_path(self, other: Path, filled: bool = ...) -> bool: ...
    def intersects_bbox(self, bbox: Bbox, filled: bool = ...): ...
    def interpolated(self, steps): ...
    def to_polygons(
        self,
        transform: Transform = ...,
        width: float = ...,
        height: float = ...,
        closed_only: bool = ...,
    ): ...
    @classmethod
    def unit_rectangle(cls) -> Path: ...
    @classmethod
    def unit_regular_polygon(cls, numVertices: int) -> Path: ...
    @classmethod
    def unit_regular_star(cls, numVertices: int, innerCircle=...) -> Path: ...
    @classmethod
    def unit_regular_asterisk(cls, numVertices: int) -> Path: ...
    @classmethod
    def unit_circle(cls) -> Path: ...
    @classmethod
    def circle(
        cls,
        center: Sequence[float] = ...,
        radius: float = ...,
        readonly: bool = ...,
    ) -> Path: ...
    @classmethod
    def unit_circle_righthalf(cls) -> Path: ...
    @classmethod
    def arc(
        cls, theta1: float, theta2: float, n: int = ..., is_wedge: bool = ...
    ) -> Path: ...
    @classmethod
    def wedge(cls, theta1: float, theta2: float, n: int = ...) -> Path: ...
    @staticmethod
    @lru_cache(8)
    def hatch(hatchpattern, density: float = ...): ...
    def clip_to_bbox(self, bbox: Bbox, inside: bool = ...): ...

def get_path_collection_extents(
    master_transform: Transform,
    paths: Sequence[Path],
    transforms: list,
    offsets,
    offset_transform: Affine2D,
) -> Bbox: ...



from typing import ClassVar, Literal, TypeVar
from scipy import sparse as sparse
from ._base import LinearModel
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numpy import ndarray
from scipy.optimize import linprog as linprog
from numbers import Real as Real
from ..base import BaseEstimator, RegressorMixin
from .._typing import Float, MatrixLike, ArrayLike
from ..utils.fixes import sp_version as sp_version, parse_version as parse_version

QuantileRegressor_Self = TypeVar("QuantileRegressor_Self", bound="QuantileRegressor")

# Authors: David Dale <dale.david@mail.ru>
#          Christian Lorentzen <lorentzen.ch@gmail.com>
# License: BSD 3 clause
import warnings

import numpy as np


class QuantileRegressor(LinearModel, RegressorMixin, BaseEstimator):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    intercept_: float = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        quantile: Float = 0.5,
        alpha: Float = 1.0,
        fit_intercept: bool = True,
        solver: Literal[
            "highs-ds",
            "highs-ipm",
            "highs",
            "interior-point",
            "revised simplex",
            "warn",
        ] = "warn",
        solver_options: None | dict = None,
    ) -> None:
        ...

    def fit(
        self: QuantileRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> QuantileRegressor_Self:
        ...



from matplotlib.backend_bases import RendererBase
from matplotlib.figure import Figure
from matplotlib._typing import Scalar

class MixedModeRenderer:
    def __init__(
        self,
        figure: Figure,
        width: Scalar,
        height: Scalar,
        dpi: float,
        vector_renderer: RendererBase,
        raster_renderer_class: RendererBase|None = None,
        bbox_inches_restore=None,
    ) -> None: ...
    def __getattr__(self, attr: str): ...
    def start_rasterizing(self) -> None: ...
    def stop_rasterizing(self) -> None: ...



from typing import Any, Callable, ClassVar, TypeVar
from copy import deepcopy as deepcopy
from ..base import BaseEstimator
from ._base import SelectorMixin
from ..exceptions import NotFittedError as NotFittedError
from ..utils._param_validation import (
    HasMethods as HasMethods,
    Interval as Interval,
    Options as Options,
)
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from ..utils.metaestimators import available_if as available_if
from ..base import clone as clone, MetaEstimatorMixin
from .._typing import MatrixLike, ArrayLike
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    check_scalar as check_scalar,
)

SelectFromModel_Self = TypeVar("SelectFromModel_Self", bound="SelectFromModel")

# Authors: Gilles Louppe, Mathieu Blondel, Maheshakya Wijewardena
# License: BSD 3 clause


import numpy as np


class SelectFromModel(MetaEstimatorMixin, SelectorMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    max_features_: int = ...
    estimator_: BaseEstimator = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: Any,
        *,
        threshold: float | None | str = None,
        prefit: bool = False,
        norm_order: float | int = 1,
        max_features: None | Callable | int = None,
        importance_getter: str | Callable = "auto",
    ) -> None:
        ...

    def fit(
        self: SelectFromModel_Self,
        X: MatrixLike,
        y: None | ArrayLike = None,
        **fit_params,
    ) -> SelectFromModel_Self:
        ...

    @property
    def threshold_(self) -> float:
        ...

    def partial_fit(
        self: SelectFromModel_Self,
        X: MatrixLike,
        y: None | ArrayLike = None,
        **fit_params,
    ) -> SelectFromModel_Self:
        ...

    @property
    def n_features_in_(self) -> int:
        ...



from ._elliptic_envelope import EllipticEnvelope as EllipticEnvelope
from ._graph_lasso import (
    graphical_lasso as graphical_lasso,
    GraphicalLasso as GraphicalLasso,
    GraphicalLassoCV as GraphicalLassoCV,
)
from ._robust_covariance import fast_mcd as fast_mcd, MinCovDet as MinCovDet
from ._empirical_covariance import (
    empirical_covariance as empirical_covariance,
    EmpiricalCovariance as EmpiricalCovariance,
    log_likelihood as log_likelihood,
)
from ._shrunk_covariance import (
    shrunk_covariance as shrunk_covariance,
    ShrunkCovariance as ShrunkCovariance,
    ledoit_wolf as ledoit_wolf,
    ledoit_wolf_shrinkage as ledoit_wolf_shrinkage,
    LedoitWolf as LedoitWolf,
    oas as oas,
    OAS as OAS,
)


__all__ = [
    "EllipticEnvelope",
    "EmpiricalCovariance",
    "GraphicalLasso",
    "GraphicalLassoCV",
    "LedoitWolf",
    "MinCovDet",
    "OAS",
    "ShrunkCovariance",
    "empirical_covariance",
    "fast_mcd",
    "graphical_lasso",
    "ledoit_wolf",
    "ledoit_wolf_shrinkage",
    "log_likelihood",
    "oas",
    "shrunk_covariance",
]



from vispy.scene.widgets.widget import Widget

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from vispy.geometry import Rect
from .widget import Widget
from .viewbox import ViewBox

from kiwisolver import Solver, Variable, UnsatisfiableConstraint

class Grid(Widget):
    def __init__(self, spacing: int = 6, **kwargs): ...
    def __getitem__(self, idxs): ...
    def add_widget(
        self,
        widget: None | Widget = None,
        row: int | None = None,
        col: int | None = None,
        row_span: int = 1,
        col_span: int = 1,
        **kwargs,
    ): ...
    def remove_widget(self, widget: Widget): ...
    def resize_widget(self, widget: Widget, row_span: int, col_span: int): ...
    def _prepare_draw(self, view): ...
    def add_grid(self, row: int | None = None, col: int | None = None, row_span: int = 1, col_span: int = 1, **kwargs): ...
    def add_view(self, row: int | None = None, col: int | None = None, row_span: int = 1, col_span: int = 1, **kwargs): ...
    def next_row(self): ...
    @property
    def grid_size(self): ...
    @property
    def layout_array(self): ...
    def __repr__(self): ...
    @staticmethod
    def _add_total_width_constraints(solver, width_grid, _var_w): ...
    @staticmethod
    def _add_total_height_constraints(solver, height_grid, _var_h): ...
    @staticmethod
    def _add_gridding_width_constraints(solver, width_grid): ...
    @staticmethod
    def _add_gridding_height_constraints(solver, height_grid): ...
    @staticmethod
    def _add_stretch_constraints(solver, width_grid, height_grid, grid_widgets, widget_grid): ...
    @staticmethod
    def _add_widget_dim_constraints(solver, width_grid, height_grid, total_var_w, total_var_h, grid_widgets): ...
    def _recreate_solver(self): ...
    def _update_child_widget_dim(self): ...
    @property
    def _widget_grid(self): ...



from typing import Any
from functools import partial

from ..classes.graph import Graph

__all__ = ["closeness_vitality"]

def closeness_vitality(
    G: Graph, node: Any = None, weight: str | None = None, wiener_index=None
) -> dict | float: ...




import numpy as np

def mean_change(arr_1: np.ndarray, arr_2: np.ndarray) -> float:
    """Calculate the mean difference between two arrays.

    Equivalent to np.abs(arr_1 - arr2).mean().
    """
    ...

def psi(x: float) -> float: ...




from typing import Mapping

from ...classes.graph import Graph

__all__ = ["average_neighbor_degree"]

def average_neighbor_degree(
    G: Graph, source="out", target="out", nodes=None, weight=None
) -> Mapping: ...



from .backend_bases import FigureManagerBase
from .figure import Figure
from typing import OrderedDict

from typing import Any, List

class Gcf:

    figs: OrderedDict = ...
    @classmethod
    def get_fig_manager(cls, num: int) -> None | FigureManagerBase: ...
    @classmethod
    def destroy(cls, num: int) -> None: ...
    @classmethod
    def destroy_fig(cls, fig: Figure)-> None: ...
    @classmethod
    def destroy_all(cls) -> None: ...
    @classmethod
    def has_fignum(cls, num: int) -> bool: ...
    @classmethod
    def get_all_fig_managers(cls) -> list[FigureManagerBase]: ...
    @classmethod
    def get_num_fig_managers(cls) -> int: ...
    @classmethod
    def get_active(cls) -> None | FigureManagerBase: ...
    @classmethod
    def set_active(cls, manager: FigureManagerBase): ...
    @classmethod
    def draw_all(cls, force: bool = False)-> None: ...



from ._plot.decision_boundary import DecisionBoundaryDisplay as DecisionBoundaryDisplay
from ._permutation_importance import permutation_importance as permutation_importance
from ._partial_dependence import partial_dependence as partial_dependence
from ._plot.partial_dependence import (
    PartialDependenceDisplay as PartialDependenceDisplay,
)


__all__ = [
    "partial_dependence",
    "permutation_importance",
    "PartialDependenceDisplay",
    "DecisionBoundaryDisplay",
]



from typing import ClassVar
from .predictor import TreePredictor
from .splitting import SplitInfo
from .common import (
    PREDICTOR_RECORD_DTYPE as PREDICTOR_RECORD_DTYPE,
    X_BITSET_INNER_DTYPE as X_BITSET_INNER_DTYPE,
    Y_DTYPE as Y_DTYPE,
    MonotonicConstraint as MonotonicConstraint,
)
from .splitting import Splitter
from numpy import ndarray
from ._bitset import (
    set_raw_bitset_from_binned_bitset as set_raw_bitset_from_binned_bitset,
)
from heapq import heappush as heappush, heappop as heappop
from .utils import sum_parallel as sum_parallel
from timeit import default_timer as time
from .histogram import HistogramBuilder
from ..._typing import Int, ArrayLike, Float, MatrixLike
import numpy as np
import numbers


EPS = ...  # to avoid zero division errors


class TreeNode:
    children_upper_bound: float = ...
    children_lower_bound: float = ...
    interaction_cst_indices: None | list[int] = ...
    allowed_features: None | ndarray = ...
    value: float | None = ...
    is_leaf: bool = ...
    sum_hessians: float = ...
    sum_gradients: float = ...
    sample_indices: ndarray = ...
    depth: int = ...

    split_info: ClassVar[None | SplitInfo] = ...
    left_child: ClassVar[TreeNode | None] = ...
    right_child: ClassVar[TreeNode | None] = ...
    histograms = ...

    # start and stop indices of the node in the splitter.partition
    # array. Concretely,
    # self.sample_indices = view(self.splitter.partition[start:stop])
    # Please see the comments about splitter.partition and
    # splitter.split_indices for more info about this design.
    # These 2 attributes are only used in _update_raw_prediction, because we
    # need to iterate over the leaves and I don't know how to efficiently
    # store the sample_indices views because they're all of different sizes.
    partition_start: ClassVar[int] = ...
    partition_stop: ClassVar[int] = ...

    def __init__(
        self,
        depth: Int,
        sample_indices: ArrayLike,
        sum_gradients: Float,
        sum_hessians: Float,
        value=None,
    ) -> None:
        ...

    def set_children_bounds(self, lower: float, upper: float) -> None:
        ...

    def __lt__(self, other_node: TreeNode) -> bool:
        ...


class TreeGrower:
    with_monotonic_cst: bool = ...
    total_apply_split_time: float = ...
    total_compute_hist_time: float = ...
    total_find_split_time: float = ...
    n_nodes: int = ...
    n_features: int = ...
    n_categorical_splits: int = ...
    missing_values_bin_idx: int = ...
    splittable_nodes: list[TreeNode] = ...
    finalized_leaves: list[TreeNode] = ...
    root: TreeNode = ...
    splitter: Splitter = ...
    histogram_builder: HistogramBuilder = ...

    def __init__(
        self,
        X_binned: MatrixLike,
        gradients: ArrayLike,
        hessians: ArrayLike,
        max_leaf_nodes: None | Int = None,
        max_depth: None | Int = None,
        min_samples_leaf: Int = 20,
        min_gain_to_split: Float = 0.0,
        n_bins: Int = 256,
        n_bins_non_missing: None | ArrayLike = None,
        has_missing_values: ndarray | ArrayLike | bool = False,
        is_categorical: None | ArrayLike = None,
        monotonic_cst: None | ArrayLike = None,
        interaction_cst: list[set[int]] | None = None,
        l2_regularization: Float = 0.0,
        min_hessian_to_split: Float = 1e-3,
        shrinkage: Float = 1.0,
        n_threads: None | Int = None,
    ) -> None:
        ...

    def grow(self) -> None:
        ...

    def split_next(self) -> tuple[TreeNode, TreeNode]:
        ...

    def make_predictor(self, binning_thresholds: ArrayLike) -> TreePredictor:
        ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from ..multiclass import (
    OneVsRestClassifier as OneVsRestClassifier,
    OneVsOneClassifier as OneVsOneClassifier,
)
from numpy.random import RandomState
from scipy.special import erf, expit as expit
from operator import itemgetter as itemgetter
from ..preprocessing import LabelEncoder as LabelEncoder
from .kernels import Kernel, Product
from ..base import BaseEstimator
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.linalg import cholesky as cholesky, cho_solve as cho_solve, solve as solve
from numbers import Integral as Integral
from ..base import ClassifierMixin, clone as clone
from .kernels import RBF as RBF, CompoundKernel as CompoundKernel, ConstantKernel as C
from .._typing import Int, MatrixLike, ArrayLike, Float
from ..utils import check_random_state as check_random_state
from ..utils.validation import check_is_fitted as check_is_fitted

_BinaryGaussianProcessClassifierLaplace_Self = TypeVar(
    "_BinaryGaussianProcessClassifierLaplace_Self",
    bound="_BinaryGaussianProcessClassifierLaplace",
)
GaussianProcessClassifier_Self = TypeVar(
    "GaussianProcessClassifier_Self", bound="GaussianProcessClassifier"
)


import numpy as np
import scipy.optimize


# Values required for approximating the logistic sigmoid by
# error functions. coefs are obtained via:
# x = np.array([0, 0.6, 2, 3.5, 4.5, np.inf])
# b = logistic(x)
# A = (erf(np.dot(x, self.lambdas)) + 1) / 2
# coefs = lstsq(A, b)[0]
LAMBDAS = ...
COEFS = ...


class _BinaryGaussianProcessClassifierLaplace(BaseEstimator):
    log_marginal_likelihood_value_: float = ...
    W_sr_: ArrayLike = ...
    pi_: ArrayLike = ...
    L_: ArrayLike = ...
    kernel_: Kernel = ...
    classes_: ArrayLike = ...
    y_train_: ArrayLike = ...
    X_train_: ArrayLike | list[Any] = ...

    def __init__(
        self,
        kernel: Product | None | Kernel = None,
        *,
        optimizer: Literal["fmin_l_bfgs_b", "fmin_l_bfgs_b"]
        | Callable = "fmin_l_bfgs_b",
        n_restarts_optimizer: Int = 0,
        max_iter_predict: Int = 100,
        warm_start: bool = False,
        copy_X_train: bool = True,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: _BinaryGaussianProcessClassifierLaplace_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
    ) -> _BinaryGaussianProcessClassifierLaplace_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def log_marginal_likelihood(
        self,
        theta: None | ArrayLike = None,
        eval_gradient: bool = False,
        clone_kernel: bool = True,
    ) -> tuple[float, ndarray] | tuple[Float, ndarray] | Float:
        ...


class GaussianProcessClassifier(ClassifierMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_classes_: int = ...
    classes_: ArrayLike = ...
    log_marginal_likelihood_value_: float = ...
    base_estimator_: BaseEstimator = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        kernel: Product | None | Kernel = None,
        *,
        optimizer: Literal["fmin_l_bfgs_b", "fmin_l_bfgs_b"]
        | None
        | Callable = "fmin_l_bfgs_b",
        n_restarts_optimizer: Int = 0,
        max_iter_predict: Int = 100,
        warm_start: bool = False,
        copy_X_train: bool = True,
        random_state: RandomState | None | Int = None,
        multi_class: Literal[
            "one_vs_rest", "one_vs_one", "one_vs_rest"
        ] = "one_vs_rest",
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit(
        self: GaussianProcessClassifier_Self, X: MatrixLike | ArrayLike, y: ArrayLike
    ) -> GaussianProcessClassifier_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    @property
    def kernel_(self) -> Product | Kernel:
        ...

    def log_marginal_likelihood(
        self,
        theta: None | ArrayLike = None,
        eval_gradient: bool = False,
        clone_kernel: bool = True,
    ) -> tuple[float, ndarray] | Float:
        ...



from typing import Any, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from ..isotonic import IsotonicRegression as IsotonicRegression
from numpy import ndarray
from ..utils._param_validation import (
    Interval as Interval,
    StrOptions as StrOptions,
    Hidden as Hidden,
)
from joblib import effective_n_jobs as effective_n_jobs
from numbers import Integral as Integral, Real as Real
from ..metrics import euclidean_distances as euclidean_distances
from ..base import BaseEstimator
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import MatrixLike, Int, Float
from ..utils import (
    check_random_state as check_random_state,
    check_array as check_array,
    check_symmetric as check_symmetric,
)

MDS_Self = TypeVar("MDS_Self", bound="MDS")


import numpy as np

import warnings


def smacof(
    dissimilarities: MatrixLike,
    *,
    metric: bool = True,
    n_components: Int = 2,
    init: None | MatrixLike = None,
    n_init: Int = 8,
    n_jobs: None | Int = None,
    max_iter: Int = 300,
    verbose: Int = 0,
    eps: Float = 1e-3,
    random_state: RandomState | None | Int = None,
    return_n_iter: bool = False,
    normalized_stress: Literal["auto", "warn"] | bool = "warn",
) -> tuple[ndarray, float, int] | tuple[ndarray, Float, int]:
    ...


class MDS(BaseEstimator):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    dissimilarity_matrix_: ndarray = ...
    stress_: float = ...
    embedding_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: Int = 2,
        *,
        metric: bool = True,
        n_init: Int = 4,
        max_iter: Int = 300,
        verbose: Int = 0,
        eps: Float = 1e-3,
        n_jobs: None | Int = None,
        random_state: RandomState | None | Int = None,
        dissimilarity: Literal["euclidean", "precomputed", "euclidean"] = "euclidean",
        normalized_stress: Literal["auto", "warn"] | bool = "warn",
    ) -> None:
        ...

    def fit(
        self: MDS_Self, X: MatrixLike, y: Any = None, init: None | MatrixLike = None
    ) -> MDS_Self:
        ...

    def fit_transform(
        self, X: MatrixLike, y: Any = None, init: None | MatrixLike = None
    ) -> ndarray:
        ...



"""
This type stub file was generated by pyright.
"""

import enum
import sys
from contextlib import contextmanager
from dataclasses import dataclass
from functools import cached_property
from typing import Dict, Iterable, Iterator, List, Literal, Optional, Sequence, Set, TYPE_CHECKING, Tuple, TypeVar, TypedDict, Union
from IPython.testing.skipdoctest import skip_doctest
from IPython.utils.decorators import sphinx_options
from IPython.utils.docs import GENERATING_DOCUMENTATION
from traitlets.config.configurable import Configurable
from typing_extensions import NotRequired, Protocol, TypeAlias, TypedDict

"""Completion for IPython.

This module started as fork of the rlcompleter module in the Python standard
library.  The original enhancements made to rlcompleter have been sent
upstream and were accepted as of Python 2.3,

This module now support a wide variety of completion mechanism both available
for normal classic Python code, as well as completer for IPython specific
Syntax like magics.

Latex and Unicode completion
============================

IPython and compatible frontends not only can complete your code, but can help
you to input a wide range of characters. In particular we allow you to insert
a unicode character using the tab completion mechanism.

Forward latex/unicode completion
--------------------------------

Forward completion allows you to easily type a unicode character using its latex
name, or unicode long description. To do so type a backslash follow by the
relevant name and press tab:


Using latex completion:

.. code::

    \\alpha<tab>
    α

or using unicode completion:


.. code::

    \\GREEK SMALL LETTER ALPHA<tab>
    α


Only valid Python identifiers will complete. Combining characters (like arrow or
dots) are also available, unlike latex they need to be put after the their
counterpart that is to say, ``F\\\\vec<tab>`` is correct, not ``\\\\vec<tab>F``.

Some browsers are known to display combining characters incorrectly.

Backward latex completion
-------------------------

It is sometime challenging to know how to type a character, if you are using
IPython, or any compatible frontend you can prepend backslash to the character
and press :kbd:`Tab` to expand it to its latex form.

.. code::

    \\α<tab>
    \\alpha


Both forward and backward completions can be deactivated by setting the
:std:configtrait:`Completer.backslash_combining_completions` option to
``False``.


Experimental
============

Starting with IPython 6.0, this module can make use of the Jedi library to
generate completions both using static analysis of the code, and dynamically
inspecting multiple namespaces. Jedi is an autocompletion and static analysis
for Python. The APIs attached to this new mechanism is unstable and will
raise unless use in an :any:`provisionalcompleter` context manager.

You will find that the following are experimental:

    - :any:`provisionalcompleter`
    - :any:`IPCompleter.completions`
    - :any:`Completion`
    - :any:`rectify_completions`

.. note::

    better name for :any:`rectify_completions` ?

We welcome any feedback on these new API, and we also encourage you to try this
module in debug mode (start IPython with ``--Completer.debug=True``) in order
to have extra logging information if :any:`jedi` is crashing, or if current
IPython completer pending deprecations are returning results not yet handled
by :any:`jedi`

Using Jedi for tab completion allow snippets like the following to work without
having to execute any code:

   >>> myvar = ['hello', 42]
   ... myvar[1].bi<tab>

Tab completion will be able to infer that ``myvar[1]`` is a real number without
executing almost any code unlike the deprecated :any:`IPCompleter.greedy`
option.

Be sure to update :any:`jedi` to the latest stable version or to try the
current development version to get better completions.

Matchers
========

All completions routines are implemented using unified *Matchers* API.
The matchers API is provisional and subject to change without notice.

The built-in matchers include:

- :any:`IPCompleter.dict_key_matcher`:  dictionary key completions,
- :any:`IPCompleter.magic_matcher`: completions for magics,
- :any:`IPCompleter.unicode_name_matcher`,
  :any:`IPCompleter.fwd_unicode_matcher`
  and :any:`IPCompleter.latex_name_matcher`: see `Forward latex/unicode completion`_,
- :any:`back_unicode_name_matcher` and :any:`back_latex_name_matcher`: see `Backward latex completion`_,
- :any:`IPCompleter.file_matcher`: paths to files and directories,
- :any:`IPCompleter.python_func_kw_matcher` - function keywords,
- :any:`IPCompleter.python_matches` - globals and attributes (v1 API),
- ``IPCompleter.jedi_matcher`` - static analysis with Jedi,
- :any:`IPCompleter.custom_completer_matcher` - pluggable completer with a default
  implementation in :any:`InteractiveShell` which uses IPython hooks system
  (`complete_command`) with string dispatch (including regular expressions).
  Differently to other matchers, ``custom_completer_matcher`` will not suppress
  Jedi results to match behaviour in earlier IPython versions.

Custom matchers can be added by appending to ``IPCompleter.custom_matchers`` list.

Matcher API
-----------

Simplifying some details, the ``Matcher`` interface can described as

.. code-block::

    MatcherAPIv1 = Callable[[str], list[str]]
    MatcherAPIv2 = Callable[[CompletionContext], SimpleMatcherResult]

    Matcher = MatcherAPIv1 | MatcherAPIv2

The ``MatcherAPIv1`` reflects the matcher API as available prior to IPython 8.6.0
and remains supported as a simplest way for generating completions. This is also
currently the only API supported by the IPython hooks system `complete_command`.

To distinguish between matcher versions ``matcher_api_version`` attribute is used.
More precisely, the API allows to omit ``matcher_api_version`` for v1 Matchers,
and requires a literal ``2`` for v2 Matchers.

Once the API stabilises future versions may relax the requirement for specifying
``matcher_api_version`` by switching to :any:`functools.singledispatch`, therefore
please do not rely on the presence of ``matcher_api_version`` for any purposes.

Suppression of competing matchers
---------------------------------

By default results from all matchers are combined, in the order determined by
their priority. Matchers can request to suppress results from subsequent
matchers by setting ``suppress`` to ``True`` in the ``MatcherResult``.

When multiple matchers simultaneously request surpression, the results from of
the matcher with higher priority will be returned.

Sometimes it is desirable to suppress most but not all other matchers;
this can be achieved by adding a set of identifiers of matchers which
should not be suppressed to ``MatcherResult`` under ``do_not_suppress`` key.

The suppression behaviour can is user-configurable via
:std:configtrait:`IPCompleter.suppress_competing_matchers`.
"""
__skip_doctest__ = ...
JEDI_INSTALLED = ...
if TYPE_CHECKING or GENERATING_DOCUMENTATION and sys.version_info >= (3, 11):
    ...
else:
    ...
if GENERATING_DOCUMENTATION:
    ...
_UNICODE_RANGES = ...
__all__ = ["Completer", "IPCompleter"]
if sys.platform == 'win32':
    PROTECTABLES = ...
else:
    ...
MATCHES_LIMIT = ...
_UNKNOWN_TYPE = ...
not_found = ...
class ProvisionalCompleterWarning(FutureWarning):
    """
    Exception raise by an experimental feature in this module.

    Wrap code in :any:`provisionalcompleter` context manager if you
    are certain you want to use an unstable feature.
    """
    ...


@skip_doctest
@contextmanager
def provisionalcompleter(action=...):
    """
    This context manager has to be used in any place where unstable completer
    behavior and API may be called.

    >>> with provisionalcompleter():
    ...     completer.do_experimental_things() # works

    >>> completer.do_experimental_things() # raises.

    .. note::

        Unstable

        By using this context manager you agree that the API in use may change
        without warning, and that you won't complain if they do so.

        You also understand that, if the API is not to your liking, you should report
        a bug to explain your use case upstream.

        We'll be happy to get your feedback, feature requests, and improvements on
        any of the unstable APIs!
    """
    ...

def has_open_quotes(s):
    """Return whether a string has open quotes.

    This simply counts whether the number of quote characters of either type in
    the string is odd.

    Returns
    -------
    If there is an open quote, the quote character is returned.  Else, return
    False.
    """
    ...

def protect_filename(s, protectables=...):
    """Escape a string to protect certain characters."""
    ...

def expand_user(path: str) -> Tuple[str, bool, str]:
    """Expand ``~``-style usernames in strings.

    This is similar to :func:`os.path.expanduser`, but it computes and returns
    extra information that will be useful if the input was being used in
    computing completions, and you wish to return the completions with the
    original '~' instead of its expanded value.

    Parameters
    ----------
    path : str
        String to be expanded.  If no ~ is present, the output is the same as the
        input.

    Returns
    -------
    newpath : str
        Result of ~ expansion in the input path.
    tilde_expand : bool
        Whether any expansion was performed or not.
    tilde_val : str
        The value that ~ was replaced with.
    """
    ...

def compress_user(path: str, tilde_expand: bool, tilde_val: str) -> str:
    """Does the opposite of expand_user, with its outputs.
    """
    ...

def completions_sorting_key(word):
    """key for sorting completions

    This does several things:

    - Demote any completions starting with underscores to the end
    - Insert any %magic and %%cellmagic completions in the alphabetical order
      by their name
    """
    ...

class _FakeJediCompletion:
    """
    This is a workaround to communicate to the UI that Jedi has crashed and to
    report a bug. Will be used only id :any:`IPCompleter.debug` is set to true.

    Added in IPython 6.0 so should likely be removed for 7.0

    """
    def __init__(self, name) -> None:
        ...
    
    def __repr__(self):
        ...
    


_JediCompletionLike = ...
class Completion:
    """
    Completion object used and returned by IPython completers.

    .. warning::

        Unstable

        This function is unstable, API may change without warning.
        It will also raise unless use in proper context manager.

    This act as a middle ground :any:`Completion` object between the
    :any:`jedi.api.classes.Completion` object and the Prompt Toolkit completion
    object. While Jedi need a lot of information about evaluator and how the
    code should be ran/inspected, PromptToolkit (and other frontend) mostly
    need user facing information.

    - Which range should be replaced replaced by what.
    - Some metadata (like completion type), or meta information to displayed to
      the use user.

    For debugging purpose we can also store the origin of the completion (``jedi``,
    ``IPython.python_matches``, ``IPython.magics_matches``...).
    """
    __slots__ = ...
    def __init__(self, start: int, end: int, text: str, *, type: Optional[str] = ..., _origin=..., signature=...) -> None:
        ...
    
    def __repr__(self):
        ...
    
    def __eq__(self, other) -> bool:
        """
        Equality and hash do not hash the type (as some completer may not be
        able to infer the type), but are use to (partially) de-duplicate
        completion.

        Completely de-duplicating completion is a bit tricker that just
        comparing as it depends on surrounding text, which Completions are not
        aware of.
        """
        ...
    
    def __hash__(self) -> int:
        ...
    


class SimpleCompletion:
    """Completion item to be included in the dictionary returned by new-style Matcher (API v2).

    .. warning::

        Provisional

        This class is used to describe the currently supported attributes of
        simple completion items, and any additional implementation details
        should not be relied on. Additional attributes may be included in
        future versions, and meaning of text disambiguated from the current
        dual meaning of "text to insert" and "text to used as a label".
    """
    __slots__ = ...
    def __init__(self, text: str, *, type: Optional[str] = ...) -> None:
        ...
    
    def __repr__(self):
        ...
    


class _MatcherResultBase(TypedDict):
    """Definition of dictionary to be returned by new-style Matcher (API v2)."""
    matched_fragment: NotRequired[str]
    suppress: NotRequired[Union[bool, Set[str]]]
    do_not_suppress: NotRequired[Set[str]]
    ordered: NotRequired[bool]
    ...


@sphinx_options(show_inherited_members=True, exclude_inherited_from=["dict"])
class SimpleMatcherResult(_MatcherResultBase, TypedDict):
    """Result of new-style completion matcher."""
    completions: Sequence[SimpleCompletion] | Iterator[SimpleCompletion]
    ...


class _JediMatcherResult(_MatcherResultBase):
    """Matching result returned by Jedi (will be processed differently)"""
    completions: Iterator[_JediCompletionLike]
    ...


AnyMatcherCompletion = ...
AnyCompletion = TypeVar("AnyCompletion", AnyMatcherCompletion, Completion)
@dataclass
class CompletionContext:
    """Completion context provided as an argument to matchers in the Matcher API v2."""
    token: str
    full_text: str
    cursor_position: int
    cursor_line: int
    limit: Optional[int]
    @cached_property
    def text_until_cursor(self) -> str:
        ...
    
    @cached_property
    def line_with_cursor(self) -> str:
        ...
    


MatcherResult = ...
class _MatcherAPIv1Base(Protocol):
    def __call__(self, text: str) -> List[str]:
        """Call signature."""
        ...
    
    __qualname__: str


class _MatcherAPIv1Total(_MatcherAPIv1Base, Protocol):
    matcher_api_version: Optional[Literal[1]]
    def __call__(self, text: str) -> List[str]:
        """Call signature."""
        ...
    


MatcherAPIv1: TypeAlias = ...
class MatcherAPIv2(Protocol):
    """Protocol describing Matcher API v2."""
    matcher_api_version: Literal[2] = ...
    def __call__(self, context: CompletionContext) -> MatcherResult:
        """Call signature."""
        ...
    
    __qualname__: str


Matcher: TypeAlias = ...
def has_any_completions(result: MatcherResult) -> bool:
    """Check if any result includes any completions."""
    ...

def completion_matcher(*, priority: Optional[float] = ..., identifier: Optional[str] = ..., api_version: int = ...): # -> (func: Unknown) -> Unknown:
    """Adds attributes describing the matcher.

    Parameters
    ----------
    priority : Optional[float]
        The priority of the matcher, determines the order of execution of matchers.
        Higher priority means that the matcher will be executed first. Defaults to 0.
    identifier : Optional[str]
        identifier of the matcher allowing users to modify the behaviour via traitlets,
        and also used to for debugging (will be passed as ``origin`` with the completions).

        Defaults to matcher function's ``__qualname__`` (for example,
        ``IPCompleter.file_matcher`` for the built-in matched defined
        as a ``file_matcher`` method of the ``IPCompleter`` class).
    api_version: Optional[int]
        version of the Matcher API used by this matcher.
        Currently supported values are 1 and 2.
        Defaults to 1.
    """
    ...

context_matcher = ...
_IC = ...
def rectify_completions(text: str, completions: _IC, *, _debug: bool = ...) -> _IC:
    """
    Rectify a set of completions to all have the same ``start`` and ``end``

    .. warning::

        Unstable

        This function is unstable, API may change without warning.
        It will also raise unless use in proper context manager.

    Parameters
    ----------
    text : str
        text that should be completed.
    completions : Iterator[Completion]
        iterator over the completions to rectify
    _debug : bool
        Log failed completion

    Notes
    -----
    :any:`jedi.api.classes.Completion` s returned by Jedi may not have the same start and end, though
    the Jupyter Protocol requires them to behave like so. This will readjust
    the completion to have the same ``start`` and ``end`` by padding both
    extremities with surrounding text.

    During stabilisation should support a ``_debug`` option to log which
    completion are return by the IPython completer and not found in Jedi in
    order to make upstream bug report.
    """
    ...

if sys.platform == 'win32':
    DELIMS = ...
else:
    ...
GREEDY_DELIMS = ...
class CompletionSplitter:
    """An object to split an input line in a manner similar to readline.

    By having our own implementation, we can expose readline-like completion in
    a uniform manner to all frontends.  This object only needs to be given the
    line of text to be split and the cursor position on said line, and it
    returns the 'word' to be completed on at the cursor after splitting the
    entire line.

    What characters are used as splitting delimiters can be controlled by
    setting the ``delims`` attribute (this is a property that internally
    automatically builds the necessary regular expression)"""
    _delims = ...
    _delim_expr = ...
    _delim_re = ...
    def __init__(self, delims=...) -> None:
        ...
    
    @property
    def delims(self):
        """Return the string of delimiter characters."""
        ...
    
    @delims.setter
    def delims(self, delims): # -> None:
        """Set the delimiters for line splitting."""
        ...
    
    def split_line(self, line, cursor_pos=...):
        """Split a line of text with a cursor at the given position.
        """
        ...
    


class Completer(Configurable):
    greedy = ...
    evaluation = ...
    use_jedi = ...
    jedi_compute_type_timeout = ...
    debug = ...
    backslash_combining_completions = ...
    auto_close_dict_keys = ...
    def __init__(self, namespace=..., global_namespace=..., **kwargs) -> None:
        """Create a new completer for the command line.

        Completer(namespace=ns, global_namespace=ns2) -> completer instance.

        If unspecified, the default namespace where completions are performed
        is __main__ (technically, __main__.__dict__). Namespaces should be
        given as dictionaries.

        An optional second namespace can be given.  This allows the completer
        to handle cases where both the local and global scopes need to be
        distinguished.
        """
        ...
    
    def complete(self, text, state): # -> None:
        """Return the next possible completion for 'text'.

        This is called successively with state == 0, 1, 2, ... until it
        returns None.  The completion should begin with 'text'.

        """
        ...
    
    def global_matches(self, text):
        """Compute matches when text is a simple name.

        Return a list of all keywords, built-in functions and names currently
        defined in self.namespace or self.global_namespace that match.

        """
        ...
    
    def attr_matches(self, text):
        """Compute matches when text contains a dot.

        Assuming the text is of the form NAME.NAME....[NAME], and is
        evaluatable in self.namespace or self.global_namespace, it will be
        evaluated and its attributes (as revealed by dir()) are used as
        possible completions.  (For class instances, class members are
        also considered.)

        WARNING: this can still invoke arbitrary C code, if an object
        with a __getattr__ hook is evaluated.

        """
        ...
    


def get__all__entries(obj):
    """returns the strings in the __all__ attribute"""
    ...

class _DictKeyState(enum.Flag):
    """Represent state of the key match in context of other possible matches.

    - given `d1 = {'a': 1}` completion on `d1['<tab>` will yield `{'a': END_OF_ITEM}` as there is no tuple.
    - given `d2 = {('a', 'b'): 1}`: `d2['a', '<tab>` will yield `{'b': END_OF_TUPLE}` as there is no tuple members to add beyond `'b'`.
    - given `d3 = {('a', 'b'): 1}`: `d3['<tab>` will yield `{'a': IN_TUPLE}` as `'a'` can be added.
    - given `d4 = {'a': 1, ('a', 'b'): 2}`: `d4['<tab>` will yield `{'a': END_OF_ITEM & END_OF_TUPLE}`
    """
    BASELINE = ...
    END_OF_ITEM = ...
    END_OF_TUPLE = ...
    IN_TUPLE = ...


_INT_FORMATS = ...
def match_dict_keys(keys: List[Union[str, bytes, Tuple[Union[str, bytes], ...]]], prefix: str, delims: str, extra_prefix: Optional[Tuple[Union[str, bytes], ...]] = ...) -> Tuple[str, int, Dict[str, _DictKeyState]]:
    """Used by dict_key_matches, matching the prefix to a list of keys

    Parameters
    ----------
    keys
        list of keys in dictionary currently being completed.
    prefix
        Part of the text already typed by the user. E.g. `mydict[b'fo`
    delims
        String of delimiters to consider when finding the current key.
    extra_prefix : optional
        Part of the text already typed in multi-key index cases. E.g. for
        `mydict['foo', "bar", 'b`, this would be `('foo', 'bar')`.

    Returns
    -------
    A tuple of three elements: ``quote``, ``token_start``, ``matched``, with
    ``quote`` being the quote that need to be used to close current string.
    ``token_start`` the position where the replacement should start occurring,
    ``matches`` a dictionary of replacement/completion keys on keys and values
        indicating whether the state.
    """
    ...

def cursor_to_position(text: str, line: int, column: int) -> int:
    """
    Convert the (line,column) position of the cursor in text to an offset in a
    string.

    Parameters
    ----------
    text : str
        The text in which to calculate the cursor offset
    line : int
        Line of the cursor; 0-indexed
    column : int
        Column of the cursor 0-indexed

    Returns
    -------
    Position of the cursor in ``text``, 0-indexed.

    See Also
    --------
    position_to_cursor : reciprocal of this function

    """
    ...

def position_to_cursor(text: str, offset: int) -> Tuple[int, int]:
    """
    Convert the position of the cursor in text (0 indexed) to a line
    number(0-indexed) and a column number (0-indexed) pair

    Position should be a valid position in ``text``.

    Parameters
    ----------
    text : str
        The text in which to calculate the cursor offset
    offset : int
        Position of the cursor in ``text``, 0-indexed.

    Returns
    -------
    (line, column) : (int, int)
        Line of the cursor; 0-indexed, column of the cursor 0-indexed

    See Also
    --------
    cursor_to_position : reciprocal of this function

    """
    ...

@context_matcher()
def back_unicode_name_matcher(context: CompletionContext): # -> SimpleMatcherResult:
    """Match Unicode characters back to Unicode name

    Same as :any:`back_unicode_name_matches`, but adopted to new Matcher API.
    """
    ...

def back_unicode_name_matches(text: str) -> Tuple[str, Sequence[str]]:
    """Match Unicode characters back to Unicode name

    This does  ``☃`` -> ``\\snowman``

    Note that snowman is not a valid python3 combining character but will be expanded.
    Though it will not recombine back to the snowman character by the completion machinery.

    This will not either back-complete standard sequences like \\n, \\b ...

    .. deprecated:: 8.6
        You can use :meth:`back_unicode_name_matcher` instead.

    Returns
    =======

    Return a tuple with two elements:

    - The Unicode character that was matched (preceded with a backslash), or
        empty string,
    - a sequence (of 1), name for the match Unicode character, preceded by
        backslash, or empty if no match.
    """
    ...

@context_matcher()
def back_latex_name_matcher(context: CompletionContext): # -> SimpleMatcherResult:
    """Match latex characters back to unicode name

    Same as :any:`back_latex_name_matches`, but adopted to new Matcher API.
    """
    ...

def back_latex_name_matches(text: str) -> Tuple[str, Sequence[str]]:
    """Match latex characters back to unicode name

    This does ``\\ℵ`` -> ``\\aleph``

    .. deprecated:: 8.6
        You can use :meth:`back_latex_name_matcher` instead.
    """
    ...

_CompleteResult = ...
DICT_MATCHER_REGEX = ...
class IPCompleter(Completer):
    """Extension of the completer class with IPython-specific features"""
    dict_keys_only = ...
    suppress_competing_matchers = ...
    merge_completions = ...
    disable_matchers = ...
    omit__names = ...
    limit_to__all__ = ...
    profile_completions = ...
    profiler_output_dir = ...
    def __init__(self, shell=..., namespace=..., global_namespace=..., config=..., **kwargs) -> None:
        """IPCompleter() -> completer

        Return a completer object.

        Parameters
        ----------
        shell
            a pointer to the ipython shell itself.  This is needed
            because this completer knows about magic functions, and those can
            only be accessed via the ipython instance.
        namespace : dict, optional
            an optional dict where completions are performed.
        global_namespace : dict, optional
            secondary optional dict for completions, to
            handle cases (such as IPython embedded inside functions) where
            both Python scopes are visible.
        config : Config
            traitlet's config object
        **kwargs
            passed to super class unmodified.
        """
        ...
    
    @property
    def matchers(self) -> List[Matcher]:
        """All active matcher routines for completion"""
        ...
    
    def all_completions(self, text: str) -> List[str]:
        """
        Wrapper around the completion methods for the benefit of emacs.
        """
        ...
    
    @context_matcher()
    def file_matcher(self, context: CompletionContext) -> SimpleMatcherResult:
        """Same as :any:`file_matches`, but adopted to new Matcher API."""
        ...
    
    def file_matches(self, text: str) -> List[str]:
        """Match filenames, expanding ~USER type strings.

        Most of the seemingly convoluted logic in this completer is an
        attempt to handle filenames with spaces in them.  And yet it's not
        quite perfect, because Python's readline doesn't expose all of the
        GNU readline details needed for this to be done correctly.

        For a filename with a space in it, the printed completions will be
        only the parts after what's already been typed (instead of the
        full completions, as is normally done).  I don't think with the
        current (as of Python 2.3) Python readline it's possible to do
        better.

        .. deprecated:: 8.6
            You can use :meth:`file_matcher` instead.
        """
        ...
    
    @context_matcher()
    def magic_matcher(self, context: CompletionContext) -> SimpleMatcherResult:
        """Match magics."""
        ...
    
    def magic_matches(self, text: str):
        """Match magics.

        .. deprecated:: 8.6
            You can use :meth:`magic_matcher` instead.
        """
        ...
    
    @context_matcher()
    def magic_config_matcher(self, context: CompletionContext) -> SimpleMatcherResult:
        """Match class names and attributes for %config magic."""
        ...
    
    def magic_config_matches(self, text: str) -> List[str]:
        """Match class names and attributes for %config magic.

        .. deprecated:: 8.6
            You can use :meth:`magic_config_matcher` instead.
        """
        ...
    
    @context_matcher()
    def magic_color_matcher(self, context: CompletionContext) -> SimpleMatcherResult:
        """Match color schemes for %colors magic."""
        ...
    
    def magic_color_matches(self, text: str) -> List[str]:
        """Match color schemes for %colors magic.

        .. deprecated:: 8.6
            You can use :meth:`magic_color_matcher` instead.
        """
        ...
    
    @completion_matcher(api_version=1)
    def python_matches(self, text: str) -> Iterable[str]:
        """Match attributes or global python names"""
        ...
    
    @context_matcher()
    def python_func_kw_matcher(self, context: CompletionContext) -> SimpleMatcherResult:
        """Match named parameters (kwargs) of the last open function."""
        ...
    
    def python_func_kw_matches(self, text):
        """Match named parameters (kwargs) of the last open function.

        .. deprecated:: 8.6
            You can use :meth:`python_func_kw_matcher` instead.
        """
        ...
    
    @context_matcher()
    def dict_key_matcher(self, context: CompletionContext) -> SimpleMatcherResult:
        """Match string keys in a dictionary, after e.g. ``foo[``."""
        ...
    
    def dict_key_matches(self, text: str) -> List[str]:
        """Match string keys in a dictionary, after e.g. ``foo[``.

        .. deprecated:: 8.6
            You can use :meth:`dict_key_matcher` instead.
        """
        ...
    
    @context_matcher()
    def unicode_name_matcher(self, context: CompletionContext): # -> SimpleMatcherResult:
        """Same as :any:`unicode_name_matches`, but adopted to new Matcher API."""
        ...
    
    @staticmethod
    def unicode_name_matches(text: str) -> Tuple[str, List[str]]:
        """Match Latex-like syntax for unicode characters base
        on the name of the character.

        This does  ``\\GREEK SMALL LETTER ETA`` -> ``η``

        Works only on valid python 3 identifier, or on combining characters that
        will combine to form a valid identifier.
        """
        ...
    
    @context_matcher()
    def latex_name_matcher(self, context: CompletionContext): # -> SimpleMatcherResult:
        """Match Latex syntax for unicode characters.

        This does both ``\\alp`` -> ``\\alpha`` and ``\\alpha`` -> ``α``
        """
        ...
    
    def latex_matches(self, text: str) -> Tuple[str, Sequence[str]]:
        """Match Latex syntax for unicode characters.

        This does both ``\\alp`` -> ``\\alpha`` and ``\\alpha`` -> ``α``

        .. deprecated:: 8.6
            You can use :meth:`latex_name_matcher` instead.
        """
        ...
    
    @context_matcher()
    def custom_completer_matcher(self, context): # -> SimpleMatcherResult:
        """Dispatch custom completer.

        If a match is found, suppresses all other matchers except for Jedi.
        """
        ...
    
    def dispatch_custom_completer(self, text):
        """
        .. deprecated:: 8.6
            You can use :meth:`custom_completer_matcher` instead.
        """
        ...
    
    def completions(self, text: str, offset: int) -> Iterator[Completion]:
        """
        Returns an iterator over the possible completions

        .. warning::

            Unstable

            This function is unstable, API may change without warning.
            It will also raise unless use in proper context manager.

        Parameters
        ----------
        text : str
            Full text of the current input, multi line string.
        offset : int
            Integer representing the position of the cursor in ``text``. Offset
            is 0-based indexed.

        Yields
        ------
        Completion

        Notes
        -----
        The cursor on a text can either be seen as being "in between"
        characters or "On" a character depending on the interface visible to
        the user. For consistency the cursor being on "in between" characters X
        and Y is equivalent to the cursor being "on" character Y, that is to say
        the character the cursor is on is considered as being after the cursor.

        Combining characters may span more that one position in the
        text.

        .. note::

            If ``IPCompleter.debug`` is :any:`True` will yield a ``--jedi/ipython--``
            fake Completion token to distinguish completion returned by Jedi
            and usual IPython completion.

        .. note::

            Completions are not completely deduplicated yet. If identical
            completions are coming from different sources this function does not
            ensure that each completion object will only be present once.
        """
        ...
    
    def complete(self, text=..., line_buffer=..., cursor_pos=...) -> Tuple[str, Sequence[str]]:
        """Find completions for the given text and line context.

        Note that both the text and the line_buffer are optional, but at least
        one of them must be given.

        Parameters
        ----------
        text : string, optional
            Text to perform the completion on.  If not given, the line buffer
            is split using the instance's CompletionSplitter object.
        line_buffer : string, optional
            If not given, the completer attempts to obtain the current line
            buffer via readline.  This keyword allows clients which are
            requesting for text completions in non-readline contexts to inform
            the completer of the entire text.
        cursor_pos : int, optional
            Index of the cursor in the full line buffer.  Should be provided by
            remote frontends where kernel has no access to frontend state.

        Returns
        -------
        Tuple of two items:
        text : str
            Text that was actually used in the completion.
        matches : list
            A list of completion matches.

        Notes
        -----
            This API is likely to be deprecated and replaced by
            :any:`IPCompleter.completions` in the future.

        """
        ...
    
    @context_matcher()
    def fwd_unicode_matcher(self, context: CompletionContext): # -> SimpleMatcherResult:
        """Same as :any:`fwd_unicode_match`, but adopted to new Matcher API."""
        ...
    
    def fwd_unicode_match(self, text: str) -> Tuple[str, Sequence[str]]:
        """
        Forward match a string starting with a backslash with a list of
        potential Unicode completions.

        Will compute list of Unicode character names on first call and cache it.

        .. deprecated:: 8.6
            You can use :meth:`fwd_unicode_matcher` instead.

        Returns
        -------
        At tuple with:
            - matched text (empty if no matches)
            - list of potential completions, empty tuple  otherwise)
        """
        ...
    
    @property
    def unicode_names(self) -> List[str]:
        """List of names of unicode code points that can be completed.

        The list is lazily initialized on first access.
        """
        ...
    





from typing import Any, Callable, ClassVar, TypeVar
from ..preprocessing import FunctionTransformer as FunctionTransformer
from ..linear_model import LinearRegression as LinearRegression
from ..exceptions import NotFittedError as NotFittedError
from numpy import ndarray
from ..utils._param_validation import HasMethods as HasMethods
from ..base import BaseEstimator, RegressorMixin, clone as clone
from .._typing import MatrixLike, ArrayLike
from ..utils import check_array as check_array
from ..utils.validation import check_is_fitted as check_is_fitted

TransformedTargetRegressor_Self = TypeVar(
    "TransformedTargetRegressor_Self", bound="TransformedTargetRegressor"
)

# Authors: Andreas Mueller <andreas.mueller@columbia.edu>
#          Guillaume Lemaitre <guillaume.lemaitre@inria.fr>
# License: BSD 3 clause

import warnings

import numpy as np

__all__ = ["TransformedTargetRegressor"]


class TransformedTargetRegressor(RegressorMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    transformer_: Any = ...
    regressor_: Any = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        regressor: Any = None,
        *,
        transformer: Any = None,
        func: None | Callable = None,
        inverse_func: None | Callable = None,
        check_inverse: bool = True,
    ) -> None:
        ...

    def fit(
        self: TransformedTargetRegressor_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        **fit_params,
    ) -> TransformedTargetRegressor_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike, **predict_params) -> ndarray:
        ...

    @property
    def n_features_in_(self) -> int:
        ...



from numpy.typing import ArrayLike
from ..classes.graph import Graph

__all__ = ["incidence_matrix", "adj_matrix", "adjacency_matrix"]

def incidence_matrix(
    G: Graph, nodelist=None, edgelist=None, oriented=False, weight=None
): ...
def adjacency_matrix(
    G: Graph, nodelist: ArrayLike | None = None, dtype=None, weight="weight"
): ...

adj_matrix = ...



import os
from typing import Any, List, Tuple, Union

from django.core.management.base import BaseCommand as BaseCommand
from django.core.management.base import CommandError as CommandError
from django.core.management.base import CommandParser as CommandParser
from django.core.management.utils import find_command as find_command
from django.core.management.utils import popen_wrapper as popen_wrapper

_PathType = Union[str, bytes, os.PathLike[Any]]

def has_bom(fn: _PathType) -> bool: ...
def is_writable(path: _PathType) -> bool: ...

class Command(BaseCommand):
    program: str = ...
    program_options: List[str] = ...
    verbosity: int = ...
    has_errors: bool = ...
    def compile_messages(
        self, locations: List[Tuple[_PathType, _PathType]]
    ) -> None: ...



from ._kmeans import (
    k_means as k_means,
    KMeans as KMeans,
    MiniBatchKMeans as MiniBatchKMeans,
    kmeans_plusplus as kmeans_plusplus,
)
from ._bicluster import (
    SpectralBiclustering as SpectralBiclustering,
    SpectralCoclustering as SpectralCoclustering,
)
from ._spectral import (
    spectral_clustering as spectral_clustering,
    SpectralClustering as SpectralClustering,
)
from ._dbscan import dbscan as dbscan, DBSCAN as DBSCAN
from ._optics import (
    OPTICS as OPTICS,
    cluster_optics_dbscan as cluster_optics_dbscan,
    compute_optics_graph as compute_optics_graph,
    cluster_optics_xi as cluster_optics_xi,
)
from ._mean_shift import (
    mean_shift as mean_shift,
    MeanShift as MeanShift,
    estimate_bandwidth as estimate_bandwidth,
    get_bin_seeds as get_bin_seeds,
)
from ._agglomerative import (
    ward_tree as ward_tree,
    AgglomerativeClustering as AgglomerativeClustering,
    linkage_tree as linkage_tree,
    FeatureAgglomeration as FeatureAgglomeration,
)
from ._bisect_k_means import BisectingKMeans as BisectingKMeans
from ._affinity_propagation import (
    affinity_propagation as affinity_propagation,
    AffinityPropagation as AffinityPropagation,
)
from ._birch import Birch as Birch

__all__ = [
    "AffinityPropagation",
    "AgglomerativeClustering",
    "Birch",
    "DBSCAN",
    "OPTICS",
    "cluster_optics_dbscan",
    "cluster_optics_xi",
    "compute_optics_graph",
    "KMeans",
    "BisectingKMeans",
    "FeatureAgglomeration",
    "MeanShift",
    "MiniBatchKMeans",
    "SpectralClustering",
    "affinity_propagation",
    "dbscan",
    "estimate_bandwidth",
    "get_bin_seeds",
    "k_means",
    "kmeans_plusplus",
    "linkage_tree",
    "mean_shift",
    "spectral_clustering",
    "ward_tree",
    "SpectralBiclustering",
    "SpectralCoclustering",
]



import numpy as np
from io import BytesIO
from enum import Enum
from functools import total_ordering
from typing import Any, Callable, Optional, Set

from matplotlib._typing import *
from matplotlib.text import Text
from matplotlib._enums import CapStyle, JoinStyle
from matplotlib.font_manager import FontProperties
from matplotlib.figure import Figure
from matplotlib.transforms import Affine2DBase, Transform
from matplotlib.backend_bases import (
    FigureCanvasBase,
    FigureManagerBase,
    GraphicsContextBase,
    _Backend,
)
from . import _backend_pdf_ps

def fill(strings, linelen: int=75)-> bytes: ...
def pdfRepr(obj) -> bytes: ...

class Reference:
    def __init__(self, id: int) -> None: ...
    def __repr__(self)-> str: ...
    def pdfRepr(self) -> bytes: ...
    def write(self, contentsy, file: "PdfFile") -> None: ...

@total_ordering
class Name:
    def __init__(self, name: bytes | Name | str) -> None: ...
    def __repr__(self)-> str: ...
    def __str__(self) -> str: ...
    def __eq__(self, other) -> bool: ...
    def __lt__(self, other) -> bool: ...
    def __hash__(self) -> int: ...
    @staticmethod
    def hexify(match)-> str: ...
    def pdfRepr(self) -> bytes: ...

class Operator:
    def __init__(self, op) -> None: ...
    def __repr__(self)-> str: ...
    def pdfRepr(self)-> bytes: ...

class Verbatim:
    def __init__(self, x: bytes) -> None: ...
    def pdfRepr(self) -> bytes: ...

class Op(Enum):
    close_fill_stroke: Op
    fill_stroke: Op
    fill: Op
    closepath: Op
    close_stroke: Op
    stroke: Op
    endpath: Op
    begin_text: Op
    end_text: Op
    curveto: Op
    rectangle: Op
    lineto: Op
    moveto: Op
    concat_matrix: Op
    use_xobject: Op
    setgray_stroke: Op
    setgray_nonstroke: Op
    setrgb_stroke: Op
    setrgb_nonstroke: Op
    setcolorspace_stroke: Op
    setcolorspace_nonstroke: Op
    setcolor_stroke: Op
    setcolor_nonstroke: Op
    setdash: Op
    setlinejoin: Op
    setlinecap: Op
    setgstate: Op
    gsave: Op
    grestore: Op
    textpos: Op
    selectfont: Op
    textmatrix : Op
    show: Op
    showkern: Op
    setlinewidth: Op
    clip: Op
    shading: Op
    op: Op
    def pdfRepr(self) -> bytes: ...
    @classmethod
    def paint_path(cls, fill: bool, stroke: bool) -> Op: ...

class Stream:
    def __init__(
        self,
        id: int,
        len: Reference | None,
        file: PdfFile,
        extra: None | dict[str, Any] = ...,
        png: None | dict[str, int] = ...,
    ) -> None: ...
    def end(self) -> None: ...
    def write(self, data: bytes) -> None: ...

class PdfFile:
    def __init__(
        self, filename: BytesIO | str | PathLike| FileLike, metadata: dict[str, Any] = ...
    ) -> None: ...
    def newPage(self, width, height)-> None: ...
    def newTextnote(self, text: str, positionRect: list[int] = ...) -> None: ...
    def finalize(self) -> None: ...
    def close(self) -> None: ...
    def write(self, data: bytes) -> None: ...
    def output(self, *data) -> None: ...
    def beginStream(
        self,
        id: int,
        len: None | Reference,
        extra: None
        | dict[str, int | Name]
        | dict[str, Name | list[int]]
        | dict[str, Name | int | Verbatim]
        | dict[str, Name | int | Verbatim | Reference] = ...,
        png: Optional[dict[str, int]] = ...,
    ) -> None: ...
    def endStream(self) -> None: ...
    def outputStream(self, ref: Reference, data: bytes, *, extra=...) -> None: ...
    def fontName(self, fontprop: FontProperties | str) -> Name: ...
    def dviFontName(self, dvifont)-> Name: ...
    def writeFonts(self) -> None: ...
    def createType1Descriptor(self, t1font, fontfile)-> Reference: ...
    def embedTTF(self, filename: str, characters: Set[int]) -> Reference: ...
    def alphaState(self, alpha: tuple[float, float]) -> Name: ...
    def writeExtGSTates(self) -> None: ...
    def hatchPattern(self, hatch_style) -> Name: ...
    def writeHatches(self) -> None: ...
    def addGouraudTriangles(
        self, points: np.ndarray, colors: np.ndarray
    ) -> tuple[Name, Reference]: ...
    def writeGouraudTriangles(self) -> None: ...
    def imageObject(self, image) -> Name: ...
    def writeImages(self) -> None: ...
    def markerObject(
        self, path, trans, fill, stroke, lw, joinstyle, capstyle
    ) -> Name: ...
    def writeMarkers(self) -> None: ...
    def pathCollectionObject(self, gc, path, trans, padding, filled, stroked)-> Name: ...
    def writePathCollectionTemplates(self) -> None: ...
    @staticmethod
    def pathOperations(path, transform, clip=..., simplify=..., sketch=...)-> list[Verbatim]: ...
    def writePath(self, path, transform, clip=..., sketch=...)-> None: ...
    def reserveObject(self, name: str = ...) -> Reference: ...
    def recordXref(self, id: int) -> None: ...
    def writeObject(self, object: Reference, contents: Any) -> None: ...
    def writeXref(self) -> None: ...
    def writeInfoDict(self) -> None: ...
    def writeTrailer(self) -> None: ...

class RendererPdf(_backend_pdf_ps.RendererPDFPSBase):
    def __init__(self, file, image_dpi, height, width) -> None: ...
    def finalize(self) -> None: ...
    def check_gc(
        self,
        gc: "GraphicsContextPdf",
        fillcolor: None | tuple[float, float, float, float] = ...,
    ) -> None: ...
    def get_image_magnification(self)-> float: ...
    def draw_image(
        self,
        gc: GraphicsContextBase,
        x: Scalar,
        y: Scalar,
        im,
        transform: Affine2DBase = ...,
    )-> None: ...
    def draw_path(self, gc, path, transform, rgbFace=...)-> None: ...
    def draw_path_collection(
        self,
        gc,
        master_transform,
        paths,
        all_transforms,
        offsets,
        offsetTrans,
        facecolors,
        edgecolors,
        linewidths,
        linestyles,
        antialiaseds,
        urls,
        offset_position,
    )-> None: ...
    def draw_markers(
        self,
        gc: GraphicsContextBase,
        marker_path,
        marker_trans: Transform,
        path,
        trans: Transform,
        rgbFace=...,
    )-> None: ...
    def draw_gouraud_triangle(self, gc: GraphicsContextBase, points, colors, trans)-> None: ...
    def draw_gouraud_triangles(self, gc, points, colors, trans)-> None: ...
    def draw_mathtext(self, gc, x, y, s, prop, angle)-> None: ...
    def draw_tex(self, gc, x, y, s, prop, angle, *, mtext=...)-> None: ...
    def encode_string(self, s: str, fonttype: int) -> bytes: ...
    def draw_text(
        self,
        gc: GraphicsContextBase,
        x: float,
        y: float,
        s: str,
        prop: FontProperties,
        angle: float,
        ismath=...,
        mtext: Text = ...,
    )-> None: ...
    def new_gc(self) -> GraphicsContextPdf: ...

class GraphicsContextPdf(GraphicsContextBase):
    def __init__(self, file: PdfFile) -> None: ...
    def __repr__(self)->str: ...
    def stroke(self) -> bool: ...
    def fill(self, *args) -> bool: ...
    def paint(
        self,
    ) -> Op: ...
    capstyles: dict[str, int] = ...
    joinstyles: dict[str, int] = ...
    def capstyle_cmd(self, style: CapStyle) -> list[int | Op]: ...
    def joinstyle_cmd(self, style: JoinStyle) -> list[int | Op]: ...
    def linewidth_cmd(self, width: int | float) -> list[float | Op]: ...
    def dash_cmd(self, dashes)-> list: ...
    def alpha_cmd(
        self, alpha: float, forced: bool, effective_alphas: tuple[float, float]
    ) -> list[Name | Op]: ...
    def hatch_cmd(
        self, hatch: None, hatch_color: tuple[float, float, float, float]
    ) -> list[Name | Op | float]: ...
    def rgb_cmd(
        self,
        rgb: tuple[float, float, float, float],
    ) -> list[float | Op]: ...
    def fillcolor_cmd(
        self,
        rgb: None | tuple[float, float, float, float],
    ) -> list[float | Op]: ...
    def push(self) -> list[Op]: ...
    def pop(self) -> list[Op]: ...
    def clip_cmd(self, cliprect, clippath) -> list[Op]: ...
    commands: tuple = ...
    def delta(self, other: "GraphicsContextPdf") -> list[Op | Name | float]: ...
    def copy_properties(self, other: GraphicsContextPdf) -> None: ...
    def finalize(self) -> list[Op]: ...

class PdfPages:
    def __init__(
        self, filename: str, keep_empty: bool = ..., metadata: dict[str, Any] = ...
    ) -> None: ...
    def __enter__(self) -> PdfPages: ...
    def __exit__(self, exc_type, exc_val, exc_tb)-> None: ...
    def close(self) -> None: ...
    def infodict(self) -> dict[str, Any]: ...
    def savefig(self, figure: Figure | int = ..., **kwargs) -> None: ...
    def get_pagecount(self) -> int: ...
    def attach_note(self, text: str, positionRect: list[int] = ...) -> None: ...

class FigureCanvasPdf(FigureCanvasBase):
    fixed_dpi: float= ...
    filetypes: dict[str, str] = ...
    def get_default_filetype(self)-> str: ...
    def print_pdf(
        self,
        filename: BytesIO | PdfPages | str,
        *,
        bbox_inches_restore=...,
        metadata=...
    ) -> None: ...
    def draw(self)-> None: ...

FigureManagerPdf = FigureManagerBase

class _BackendPdf(_Backend):
    FigureCanvas = FigureCanvasPdf



__all__ = ["nonisomorphic_trees", "number_of_nonisomorphic_trees"]

from ..classes.graph import Graph

def nonisomorphic_trees(order: int, create="graph"): ...
def number_of_nonisomorphic_trees(order: int): ...



from typing import Mapping
from numpy.typing import ArrayLike
from ...classes.graph import Graph

__all__ = [
    "floyd_warshall",
    "floyd_warshall_predecessor_and_distance",
    "reconstruct_path",
    "floyd_warshall_numpy",
]

def floyd_warshall_numpy(G: Graph, nodelist=None, weight="weight"): ...
def floyd_warshall_predecessor_and_distance(G: Graph, weight="weight") -> dict: ...
def reconstruct_path(source, target, predecessors: Mapping) -> ArrayLike: ...
def floyd_warshall(G: Graph, weight="weight") -> Mapping: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.csgraph._reordering, version: unspecified
import typing
import builtins as _mod_builtins
import numpy as _mod_numpy
import scipy.sparse.base as _mod_scipy_sparse_base
import scipy.sparse.csc as _mod_scipy_sparse_csc
import scipy.sparse.csr as _mod_scipy_sparse_csr

DTYPE = _mod_numpy.float64
ITYPE = _mod_numpy.int32
SparseEfficiencyWarning = _mod_scipy_sparse_base.SparseEfficiencyWarning
__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _reverse_cuthill_mckee(ind, ptr, num_rows) -> typing.Any:
    '\n    Reverse Cuthill-McKee ordering of a sparse symmetric CSR or CSC matrix.  \n    We follow the original Cuthill-McKee paper and always start the routine\n    at a node of lowest degree for each connected component.\n    '
    ...

csc_matrix = _mod_scipy_sparse_csc.csc_matrix
csr_matrix = _mod_scipy_sparse_csr.csr_matrix
def isspmatrix(x) -> typing.Any:
    'Is x of a sparse matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a sparse matrix\n\n    Returns\n    -------\n    bool\n        True if x is a sparse matrix, False otherwise\n\n    Notes\n    -----\n    issparse and isspmatrix are aliases for the same function.\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix\n    >>> isspmatrix(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import isspmatrix\n    >>> isspmatrix(5)\n    False\n    '
    ...

def isspmatrix_coo(x) -> typing.Any:
    'Is x of coo_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a coo matrix\n\n    Returns\n    -------\n    bool\n        True if x is a coo matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import coo_matrix, isspmatrix_coo\n    >>> isspmatrix_coo(coo_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import coo_matrix, csr_matrix, isspmatrix_coo\n    >>> isspmatrix_coo(csr_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_csc(x) -> typing.Any:
    'Is x of csc_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csc matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csc matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csc_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csc_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csr_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_csr(x) -> typing.Any:
    'Is x of csr_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csr matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csr matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix_csr\n    >>> isspmatrix_csr(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csr(csc_matrix([[5]]))\n    False\n    '
    ...

def maximum_bipartite_matching(graph, perm_type=...) -> typing.Any:
    '\n    maximum_bipartite_matching(graph, perm_type=\'row\')\n\n    Returns a matching of a bipartite graph whose cardinality is as least that\n    of any given matching of the graph.\n\n    Parameters\n    ----------\n    graph : sparse matrix\n        Input sparse in CSR format whose rows represent one partition of the\n        graph and whose columns represent the other partition. An edge between\n        two vertices is indicated by the corresponding entry in the matrix\n        existing in its sparse representation.\n    perm_type : str, {\'row\', \'column\'}\n        Which partition to return the matching in terms of: If ``\'row\'``, the\n        function produces an array whose length is the number of columns in the\n        input, and whose :math:`j`\'th element is the row matched to the\n        :math:`j`\'th column. Conversely, if ``perm_type`` is ``\'column\'``, this\n        returns the columns matched to each row.\n\n    Returns\n    -------\n    perm : ndarray\n        A matching of the vertices in one of the two partitions. Unmatched\n        vertices are represented by a ``-1`` in the result.\n\n    Notes\n    -----\n    This function implements the Hopcroft--Karp algorithm [1]_. Its time\n    complexity is :math:`O(\\lvert E \\rvert \\sqrt{\\lvert V \\rvert})`, and its\n    space complexity is linear in the number of rows. In practice, this\n    asymmetry between rows and columns means that it can be more efficient to\n    transpose the input if it contains more columns than rows.\n\n    By Konig\'s theorem, the cardinality of the matching is also the number of\n    vertices appearing in a minimum vertex cover of the graph.\n\n    Note that if the sparse representation contains explicit zeros, these are\n    still counted as edges.\n\n    The implementation was changed in SciPy 1.4.0 to allow matching of general\n    bipartite graphs, where previous versions would assume that a perfect\n    matching existed. As such, code written against 1.4.0 will not necessarily\n    work on older versions.\n\n    References\n    ----------\n    .. [1] John E. Hopcroft and Richard M. Karp. "An n^{5 / 2} Algorithm for\n           Maximum Matchings in Bipartite Graphs" In: SIAM Journal of Computing\n           2.4 (1973), pp. 225--231. :doi:`10.1137/0202019`\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import maximum_bipartite_matching\n\n    As a simple example, consider a bipartite graph in which the partitions\n    contain 2 and 3 elements respectively. Suppose that one partition contains\n    vertices labelled 0 and 1, and that the other partition contains vertices\n    labelled A, B, and C. Suppose that there are edges connecting 0 and C,\n    1 and A, and 1 and B. This graph would then be represented by the following\n    sparse matrix:\n\n    >>> graph = csr_matrix([[0, 0, 1], [1, 1, 0]])\n\n    Here, the 1s could be anything, as long as they end up being stored as\n    elements in the sparse matrix. We can now calculate maximum matchings as\n    follows:\n\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'column\'))\n    [2 0]\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'row\'))\n    [ 1 -1  0]\n\n    The first output tells us that 1 and 2 are matched with C and A\n    respectively, and the second output tells us that A, B, and C are matched\n    with 1, nothing, and 0 respectively.\n\n    Note that explicit zeros are still converted to edges. This means that a\n    different way to represent the above graph is by using the CSR structure\n    directly as follows:\n\n    >>> data = [0, 0, 0]\n    >>> indices = [2, 0, 1]\n    >>> indptr = [0, 1, 3]\n    >>> graph = csr_matrix((data, indices, indptr))\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'column\'))\n    [2 0]\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'row\'))\n    [ 1 -1  0]\n\n    When one or both of the partitions are empty, the matching is empty as\n    well:\n\n    >>> graph = csr_matrix((2, 0))\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'column\'))\n    [-1 -1]\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'row\'))\n    []\n\n    When the input matrix is square, and the graph is known to admit a perfect\n    matching, i.e. a matching with the property that every vertex in the graph\n    belongs to some edge in the matching, then one can view the output as the\n    permutation of rows (or columns) turning the input matrix into one with the\n    property that all diagonal elements are non-empty:\n\n    >>> a = [[0, 1, 2, 0], [1, 0, 0, 1], [2, 0, 0, 3], [0, 1, 3, 0]]\n    >>> graph = csr_matrix(a)\n    >>> perm = maximum_bipartite_matching(graph, perm_type=\'row\')\n    >>> print(graph[perm].toarray())\n    [[1 0 0 1]\n     [0 1 2 0]\n     [0 1 3 0]\n     [2 0 0 3]]\n\n    '
    ...

def reverse_cuthill_mckee(graph, symmetric_mode=...) -> typing.Any:
    '\n    reverse_cuthill_mckee(graph, symmetric_mode=False)\n    \n    Returns the permutation array that orders a sparse CSR or CSC matrix\n    in Reverse-Cuthill McKee ordering.  \n    \n    It is assumed by default, ``symmetric_mode=False``, that the input matrix \n    is not symmetric and works on the matrix ``A+A.T``. If you are \n    guaranteed that the matrix is symmetric in structure (values of matrix \n    elements do not matter) then set ``symmetric_mode=True``.\n    \n    Parameters\n    ----------\n    graph : sparse matrix\n        Input sparse in CSC or CSR sparse matrix format.\n    symmetric_mode : bool, optional\n        Is input matrix guaranteed to be symmetric.\n\n    Returns\n    -------\n    perm : ndarray\n        Array of permuted row and column indices.\n \n    Notes\n    -----\n    .. versionadded:: 0.15.0\n\n    References\n    ----------\n    E. Cuthill and J. McKee, "Reducing the Bandwidth of Sparse Symmetric Matrices",\n    ACM \'69 Proceedings of the 1969 24th national conference, (1969).\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import reverse_cuthill_mckee\n\n    >>> graph = [\n    ... [0, 1 , 2, 0],\n    ... [0, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 0, 0, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 3)\t1\n      (2, 0)\t2\n      (2, 3)\t3\n\n    >>> reverse_cuthill_mckee(graph)\n    array([3, 2, 1, 0], dtype=int32)\n    \n    '
    ...

def structural_rank(graph) -> typing.Any:
    '\n    structural_rank(graph)\n    \n    Compute the structural rank of a graph (matrix) with a given \n    sparsity pattern.\n\n    The structural rank of a matrix is the number of entries in the maximum \n    transversal of the corresponding bipartite graph, and is an upper bound \n    on the numerical rank of the matrix. A graph has full structural rank \n    if it is possible to permute the elements to make the diagonal zero-free.\n\n    .. versionadded:: 0.19.0\n\n    Parameters\n    ----------\n    graph : sparse matrix\n        Input sparse matrix.\n\n    Returns\n    -------\n    rank : int\n        The structural rank of the sparse graph.\n    \n    References\n    ----------\n    .. [1] I. S. Duff, "Computing the Structural Index", SIAM J. Alg. Disc. \n            Meth., Vol. 7, 594 (1986).\n    \n    .. [2] http://www.cise.ufl.edu/research/sparse/matrices/legend.html\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import structural_rank\n\n    >>> graph = [\n    ... [0, 1, 2, 0],\n    ... [1, 0, 0, 1],\n    ... [2, 0, 0, 3],\n    ... [0, 1, 3, 0]\n    ... ]\n    >>> graph = csr_matrix(graph)\n    >>> print(graph)\n      (0, 1)\t1\n      (0, 2)\t2\n      (1, 0)\t1\n      (1, 3)\t1\n      (2, 0)\t2\n      (2, 3)\t3\n      (3, 1)\t1\n      (3, 2)\t3\n\n    >>> structural_rank(graph)\n    4\n\n    '
    ...

def warn(message, category, stacklevel, source) -> typing.Any:
    'Issue a warning, or maybe ignore it or raise an exception.'
    ...

def __getattr__(name) -> typing.Any:
    ...




# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
from .raw_polygon_collection import RawPolygonCollection

# from . agg_polygon_collection import AggPolygonCollection
# from . agg_fast_polygon_collection import AggPolygonCollection

def PolygonCollection(mode="raw", *args, **kwargs): ...



from django.core.handlers.wsgi import WSGIRequest
from django.http.response import HttpResponse
from django.utils.deprecation import MiddlewareMixin

class FlatpageFallbackMiddleware(MiddlewareMixin):
    def process_response(
        self, request: WSGIRequest, response: HttpResponse
    ) -> HttpResponse: ...



from .._typing import Scalar
from numpy.typing import NDArray, ArrayLike
from typing import Sequence

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import warnings

import numpy as np

from .color_array import ColorArray
from ..ext.cubehelix import cubehelix
from ..util.check_environment import has_matplotlib
import vispy.gloo

###############################################################################
# Color maps

# Length of the texture map used for luminance to RGBA conversion
LUT_len: int = ...

# Utility functions for interpolation in NumPy.
def _vector_or_scalar(x, type="row"): ...
def _vector(x, type="row"): ...
def _find_controls(x, controls=None, clip=None): ...

# Normalization
def _normalize(x, cmin=None, cmax=None, clip=True): ...

# Interpolation functions in NumPy.
def _mix_simple(a, b, x): ...
def _interpolate_multi(colors, x, controls): ...
def mix(colors, x, controls=None): ...
def smoothstep(edge0, edge1, x): ...
def step(colors, x, controls=None): ...

# GLSL interpolation functions.
def _glsl_mix(controls=None, colors=None, texture_map_data=None): ...
def _glsl_step(controls=None, colors=None, texture_map_data=None): ...

# Mini GLSL template system for colors.
def _process_glsl_template(template, colors): ...

class BaseColormap(object):

    # Control colors used by the colormap.
    colors: None = ...

    # GLSL string with a function implementing the color map.
    glsl_map: None = ...

    # Texture map data used by the 'colormap' GLSL function
    # for luminance to RGBA conversion.
    texture_map_data: None = ...

    def __init__(self, colors: Sequence[ArrayLike | tuple | NDArray] | None = None): ...
    def map(self, item: NDArray) -> NDArray: ...
    def texture_lut(self): ...
    def __getitem__(self, item): ...
    def __setitem__(self, item, value): ...
    def _repr_html_(self): ...

def _default_controls(ncolors): ...

# List the parameters of every supported interpolation mode.
_interpolation_info: dict = ...

class Colormap(BaseColormap):
    def __init__(
        self,
        colors: ArrayLike,
        controls: ArrayLike | None = None,
        interpolation: str = "linear",
    ): ...
    @property
    def interpolation(self): ...
    @interpolation.setter
    def interpolation(self, val): ...
    def map(self, x: ArrayLike) -> ArrayLike: ...
    def texture_lut(self): ...

class MatplotlibColormap(Colormap):
    def __init__(self, name: str): ...

class CubeHelixColormap(Colormap):
    def __init__(
        self,
        start: Scalar = 0.5,
        rot: Scalar = 1,
        gamma: Scalar = 1.0,
        reverse: bool = True,
        nlev: Scalar = 32,
        minSat: Scalar = 1.2,
        maxSat: Scalar = 1.2,
        minLight: Scalar = 0.0,
        maxLight: Scalar = 1.0,
        **kwargs,
    ): ...

class _Fire(BaseColormap):
    colors: list = ...

    glsl_map: str = ...

    def map(self, t) -> NDArray: ...

class _Grays(BaseColormap):
    glsl_map: str = ...

    def map(self, t) -> NDArray: ...

class _Ice(BaseColormap):
    glsl_map: str = ...

    def map(self, t) -> NDArray: ...

class _Hot(BaseColormap):
    colors: list = ...

    glsl_map: str = ...

    def map(self, t) -> NDArray: ...

class _Winter(BaseColormap):
    colors: list = ...

    glsl_map: str = ...

    def map(self, t) -> NDArray: ...

class SingleHue(Colormap):
    def __init__(
        self,
        hue: Scalar = 200,
        saturation_range: ArrayLike = [0.1, 0.8],
        value: Scalar = 1.0,
    ): ...

class HSL(Colormap):
    def __init__(
        self,
        ncolors=6,
        hue_start: int = 0,
        saturation: float = 1.0,
        value: float = 1.0,
        controls: ArrayLike | None = None,
        interpolation: str = "linear",
    ): ...

class HSLuv(Colormap):
    def __init__(
        self,
        ncolors=6,
        hue_start: int = 0,
        saturation: float = 1.0,
        value: float = 0.7,
        controls: ArrayLike | None = None,
        interpolation: str = "linear",
    ): ...

class _HUSL(HSLuv):
    def __init__(self, *args, **kwargs): ...

class Diverging(Colormap):
    def __init__(self, h_pos=20, h_neg=250, saturation=1.0, value=0.7, center="light"): ...

class RedYellowBlueCyan(Colormap):
    def __init__(self, limits: ArrayLike = ...): ...

# https://github.com/matplotlib/matplotlib/pull/4707/files#diff-893cf0348279e9f4570488a7a297ab1eR774  # noqa
# Taken from original Viridis colormap data in matplotlib implementation
#
# Issue #1331 https://github.com/vispy/vispy/issues/1331 explains that the
# 128 viridis sample size fails on some GPUs
# but lowering to 64 samples allows more GPUs to use viridis.
#
# VisPy has beem updated to use a texture map lookup.
# Thus, sampling of the Viridis colormap data is no longer necessary.
_viridis_data: list = ...

_colormaps = ...

def get_colormap(name: str | Colormap, *args, **kwargs): ...
def get_colormaps(): ...



from typing import Mapping
from ...classes.graph import Graph

__all__ = ["cytoscape_data", "cytoscape_graph"]

def cytoscape_data(
    G: Graph, attrs=None, name: str = "name", ident: str = "id"
) -> Mapping: ...
def cytoscape_graph(
    data: Mapping, attrs=None, name: str = "name", ident: str = "id"
): ...



from typing import Any, ClassVar, Iterable, Literal, TypeVar
from scipy import linalg as linalg
from ..linear_model import lars_path_gram as lars_path_gram
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ..model_selection._split import BaseShuffleSplit
from numbers import Integral as Integral, Real as Real
from ..model_selection import check_cv as check_cv, cross_val_score as cross_val_score
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import MatrixLike, Float, Int, ArrayLike
from ..model_selection import BaseCrossValidator
from ..utils.validation import (
    check_random_state as check_random_state,
    check_scalar as check_scalar,
)
from . import (
    empirical_covariance as empirical_covariance,
    EmpiricalCovariance,
    log_likelihood as log_likelihood,
)

GraphicalLassoCV_Self = TypeVar("GraphicalLassoCV_Self", bound="GraphicalLassoCV")
GraphicalLasso_Self = TypeVar("GraphicalLasso_Self", bound="GraphicalLasso")


# Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
# License: BSD 3 clause
# Copyright: INRIA
import warnings
import operator
import sys
import time
import numpy as np


def alpha_max(emp_cov: MatrixLike) -> Float:
    ...


# The g-lasso algorithm
def graphical_lasso(
    emp_cov: MatrixLike,
    alpha: Float,
    *,
    cov_init: None | MatrixLike = None,
    mode: Literal["cd", "lars", "cd"] = "cd",
    tol: Float = 1e-4,
    enet_tol: Float = 1e-4,
    max_iter: Int = 100,
    verbose: bool = False,
    return_costs: bool = False,
    eps: Float = ...,
    return_n_iter: bool = False,
) -> tuple[ndarray, ndarray, list[tuple], int]:
    ...


class BaseGraphicalLasso(EmpiricalCovariance):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    precision_: ndarray = ...
    covariance_: ndarray = ...
    location_: ndarray = ...
    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        tol: float = 1e-4,
        enet_tol: float = 1e-4,
        max_iter: int = 100,
        mode: str = "cd",
        verbose: bool = False,
        assume_centered: bool = False,
    ) -> None:
        ...


class GraphicalLasso(BaseGraphicalLasso):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    precision_: ndarray = ...
    covariance_: ndarray = ...
    location_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        alpha: Float = 0.01,
        *,
        mode: Literal["cd", "lars", "cd"] = "cd",
        tol: Float = 1e-4,
        enet_tol: Float = 1e-4,
        max_iter: Int = 100,
        verbose: bool = False,
        assume_centered: bool = False,
    ) -> None:
        ...

    def fit(
        self: GraphicalLasso_Self, X: MatrixLike, y: Any = None
    ) -> GraphicalLasso_Self:
        ...


# Cross-validation with GraphicalLasso
def graphical_lasso_path(
    X: ArrayLike,
    alphas: ArrayLike,
    cov_init: None | MatrixLike = None,
    X_test: None | MatrixLike = None,
    mode: Literal["cd", "lars", "cd"] = "cd",
    tol: Float = 1e-4,
    enet_tol: Float = 1e-4,
    max_iter: Int = 100,
    verbose: int | bool = False,
) -> tuple[list[ndarray], list[ndarray], list[Float]] | tuple[
    list[ndarray], list[ndarray], list[float]
]:
    ...


class GraphicalLassoCV(BaseGraphicalLasso):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: int = ...
    cv_results_: dict[str, ndarray] = ...
    alpha_: float = ...
    precision_: ndarray = ...
    covariance_: ndarray = ...
    location_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        alphas: ArrayLike | int = 4,
        n_refinements: Int = 4,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        tol: Float = 1e-4,
        enet_tol: Float = 1e-4,
        max_iter: Int = 100,
        mode: Literal["cd", "lars", "cd"] = "cd",
        n_jobs: None | Int = None,
        verbose: bool = False,
        assume_centered: bool = False,
    ) -> None:
        ...

    def fit(
        self: GraphicalLassoCV_Self, X: MatrixLike, y: Any = None
    ) -> GraphicalLassoCV_Self:
        ...



# graph drawing and interface to graphviz

from .layout import (
    bipartite_layout as bipartite_layout,
    circular_layout as circular_layout,
    kamada_kawai_layout as kamada_kawai_layout,
    random_layout as random_layout,
    rescale_layout as rescale_layout,
    rescale_layout_dict as rescale_layout_dict,
    shell_layout as shell_layout,
    spring_layout as spring_layout,
    spectral_layout as spectral_layout,
    planar_layout as planar_layout,
    fruchterman_reingold_layout as fruchterman_reingold_layout,
    spiral_layout as spiral_layout,
    multipartite_layout as multipartite_layout,
)
from .nx_pylab import (
    draw as draw,
    draw_networkx as draw_networkx,
    draw_networkx_nodes as draw_networkx_nodes,
    draw_networkx_edges as draw_networkx_edges,
    draw_networkx_labels as draw_networkx_labels,
    draw_networkx_edge_labels as draw_networkx_edge_labels,
    draw_circular as draw_circular,
    draw_kamada_kawai as draw_kamada_kawai,
    draw_random as draw_random,
    draw_spectral as draw_spectral,
    draw_spring as draw_spring,
    draw_planar as draw_planar,
    draw_shell as draw_shell,
)
from . import nx_agraph as nx_agraph
from . import nx_pydot as nx_pydot



from typing import Any, ClassVar, Iterable, TypeVar
from ._config import get_config as get_config
from .utils._set_output import _SetOutputMixin
from .utils._estimator_html_repr import estimator_html_repr as estimator_html_repr
from collections import defaultdict as defaultdict
from .utils.validation import (
    check_X_y as check_X_y,
    check_array as check_array,
    check_is_fitted as check_is_fitted,
)
from .metrics import accuracy_score as accuracy_score, r2_score as r2_score
from numpy import ndarray
from .utils._param_validation import (
    validate_parameter_constraints as validate_parameter_constraints,
)
from ._typing import MatrixLike, ArrayLike, Float, Int

BaseEstimator_Self = TypeVar("BaseEstimator_Self", bound="BaseEstimator")


# Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
# License: BSD 3 clause

import copy
import warnings
import platform
import inspect
import re

import numpy as np


def clone(
    estimator: BaseEstimator | Iterable[BaseEstimator], *, safe: bool = True
) -> Any:
    ...


class BaseEstimator:
    def get_params(self, deep: bool = True) -> dict:
        ...

    def set_params(self: BaseEstimator_Self, **params) -> BaseEstimator_Self:
        ...

    def __repr__(self, N_CHAR_MAX: int = 700) -> str:
        ...

    def __getstate__(self):
        ...

    def __setstate__(self, state) -> None:
        ...


class ClassifierMixin:

    _estimator_type: ClassVar[str] = ...

    def score(
        self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...


class RegressorMixin:

    _estimator_type: ClassVar[str] = ...

    def score(
        self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...


class ClusterMixin:

    _estimator_type: ClassVar[str] = ...

    def fit_predict(self, X: MatrixLike, y: Any = None) -> ndarray:
        ...


class BiclusterMixin:
    def biclusters_(self):
        ...

    def get_indices(self, i: Int) -> tuple[ndarray, ndarray]:
        ...

    def get_shape(self, i: Int) -> tuple[int, int]:
        ...

    def get_submatrix(self, i: Int, data: MatrixLike) -> ndarray:
        ...


class TransformerMixin(_SetOutputMixin):
    def fit_transform(
        self, X: MatrixLike, y: None | MatrixLike | ArrayLike = None, **fit_params
    ) -> ndarray:
        ...


class OneToOneFeatureMixin:
    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...


class ClassNamePrefixFeaturesOutMixin:
    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...


class DensityMixin:

    _estimator_type: ClassVar[str] = ...

    def score(self, X: MatrixLike, y: Any = None) -> float:
        ...


class OutlierMixin:

    _estimator_type: ClassVar[str] = ...

    def fit_predict(self, X: MatrixLike | ArrayLike, y: Any = None) -> ndarray:
        ...


class MetaEstimatorMixin:
    _required_parameters: ClassVar[list] = ...


class MultiOutputMixin:
    ...


class _UnstableArchMixin:
    ...


def is_classifier(estimator: Any) -> bool:
    ...


def is_regressor(estimator: BaseEstimator) -> bool:
    ...


def is_outlier_detector(estimator: BaseEstimator) -> bool:
    ...



"""
This type stub file was generated by pyright.
"""

from traitlets.config.configurable import Configurable

"""An interface for publishing rich data to frontends.

There are two components of the display system:

* Display formatters, which take a Python object and compute the
  representation of the object in various formats (text, HTML, SVG, etc.).
* The display publisher that is used to send the representation data to the
  various frontends.

This module defines the logic display publishing. The display publisher uses
the ``display_data`` message type that is defined in the IPython messaging
spec.
"""
class DisplayPublisher(Configurable):
    """A traited class that publishes display data to frontends.

    Instances of this class are created by the main IPython object and should
    be accessed there.
    """
    def __init__(self, shell=..., *args, **kwargs) -> None:
        ...
    
    def publish(self, data, metadata=..., source=..., *, transient=..., update=..., **kwargs) -> None:
        """Publish data and metadata to all frontends.

        See the ``display_data`` message in the messaging documentation for
        more details about this message type.

        The following MIME types are currently implemented:

        * text/plain
        * text/html
        * text/markdown
        * text/latex
        * application/json
        * application/javascript
        * image/png
        * image/jpeg
        * image/svg+xml

        Parameters
        ----------
        data : dict
            A dictionary having keys that are valid MIME types (like
            'text/plain' or 'image/svg+xml') and values that are the data for
            that MIME type. The data itself must be a JSON'able data
            structure. Minimally all data should have the 'text/plain' data,
            which can be displayed by all frontends. If more than the plain
            text is given, it is up to the frontend to decide which
            representation to use.
        metadata : dict
            A dictionary for metadata related to the data. This can contain
            arbitrary key, value pairs that frontends can use to interpret
            the data.  Metadata specific to each mime-type can be specified
            in the metadata dict with the same mime-type keys as
            the data itself.
        source : str, deprecated
            Unused.
        transient : dict, keyword-only
            A dictionary for transient data.
            Data in this dictionary should not be persisted as part of saving this output.
            Examples include 'display_id'.
        update : bool, keyword-only, default: False
            If True, only update existing outputs with the same display_id,
            rather than creating a new output.
        """
        ...
    
    def clear_output(self, wait=...): # -> None:
        """Clear the output of the cell receiving output."""
        ...
    


class CapturingDisplayPublisher(DisplayPublisher):
    """A DisplayPublisher that stores"""
    outputs = ...
    def publish(self, data, metadata=..., source=..., *, transient=..., update=...): # -> None:
        ...
    
    def clear_output(self, wait=...): # -> None:
        ...
    






import numpy as np
from scipy.sparse import csr_matrix, csc_matrix


def csr_row_norms(X: np.ndarray) -> np.ndarray:
    """Squared L2 norm of each row in CSR matrix X."""
    ...


def csr_mean_variance_axis0(X: csr_matrix, weights:np.ndarray|None=None, return_sum_weights:bool=False) ->     tuple[np.ndarray, np.ndarray, np.ndarray]:
    """Compute mean and variance along axis 0 on a CSR matrix
    Uses a np.float64 accumulator.
    Parameters
    ----------
    X : CSR sparse matrix, shape (n_samples, n_features)
        Input data.
    weights : ndarray of shape (n_samples,), dtype=floating, default=None
        If it is set to None samples will be equally weighted.
        .. versionadded:: 0.24
    return_sum_weights : bool, default=False
        If True, returns the sum of weights seen for each feature.
        .. versionadded:: 0.24
    Returns
    -------
    means : float array with shape (n_features,)
        Feature-wise means
    variances : float array with shape (n_features,)
        Feature-wise variances
    sum_weights : ndarray of shape (n_features,), dtype=floating
        Returned if return_sum_weights is True.
    """
    ...


def csc_mean_variance_axis0(X: csc_matrix, weights: np.ndarray|None=None, return_sum_weights: bool=False) ->     tuple[np.ndarray, np.ndarray, np.ndarray]:
    """Compute mean and variance along axis 0 on a CSC matrix
    Uses a np.float64 accumulator.
    Parameters
    ----------
    X : CSC sparse matrix, shape (n_samples, n_features)
        Input data.
    weights : ndarray of shape (n_samples,), dtype=floating, default=None
        If it is set to None samples will be equally weighted.
        .. versionadded:: 0.24
    return_sum_weights : bool, default=False
        If True, returns the sum of weights seen for each feature.
        .. versionadded:: 0.24
    Returns
    -------
    means : float array with shape (n_features,)
        Feature-wise means
    variances : float array with shape (n_features,)
        Feature-wise variances
    sum_weights : ndarray of shape (n_features,), dtype=floating
        Returned if return_sum_weights is True.
    """
    ...

        
def incr_mean_variance_axis0(X: csr_matrix|csc_matrix, last_mean: np.ndarray, last_var: np.ndarray,         last_n: np.ndarray, weights: np.ndarray|None=None) -> tuple[np.ndarray, np.ndarray, np.ndarray]:
    """Compute mean and variance along axis 0 on a CSR or CSC matrix.
    last_mean, last_var are the statistics computed at the last step by this
    function. Both must be initialized to 0.0. last_n is the
    number of samples encountered until now and is initialized at 0.
    Parameters
    ----------
    X : CSR or CSC sparse matrix, shape (n_samples, n_features)
      Input data.
    last_mean : float array with shape (n_features,)
      Array of feature-wise means to update with the new data X.
    last_var : float array with shape (n_features,)
      Array of feature-wise var to update with the new data X.
    last_n : float array with shape (n_features,)
      Sum of the weights seen so far (if weights are all set to 1
      this will be the same as number of samples seen so far, before X).
    weights : float array with shape (n_samples,) or None. If it is set
      to None samples will be equall weighted.
    Returns
    -------
    updated_mean : float array with shape (n_features,)
      Feature-wise means
    updated_variance : float array with shape (n_features,)
      Feature-wise variances
    updated_n : int array with shape (n_features,)
      Updated number of samples seen
    """
    ...

    
def inplace_csr_row_normalize_l1(X: np.ndarray) -> None:
    """Inplace row normalize using the l1 norm"""
    ...

    
def inplace_csr_row_normalize_l2(X: np.ndarray) -> None:
    """Inplace row normalize using the l2 norm"""
    ...


def assign_rows_csr(
    X: csr_matrix,
    X_rows: np.ndarray,
    out_rows: np.ndarray,
    out: np.ndarray,
) -> None:
    """Densify selected rows of a CSR matrix into a preallocated array.
    Like out[out_rows] = X[X_rows].toarray() but without copying.
    No-copy supported for both dtype=np.float32 and dtype=np.float64.
    Parameters
    ----------
    X : scipy.sparse.csr_matrix, shape=(n_samples, n_features)
    X_rows : array, dtype=np.intp, shape=n_rows
    out_rows : array, dtype=np.intp, shape=n_rows
    out : array, shape=(arbitrary, n_features)
    """   
    ...
